@@ -42,10 +42,14 @@
 #include "qplatformdefs.h"
 #include "qfilesystemengine_p.h"
 #include "qfile.h"
+#include "qstorageinfo.h"
 
 #include <QtCore/qoperatingsystemversion.h>
 #include <QtCore/private/qcore_unix_p.h>
 #include <QtCore/qvarlengtharray.h>
+#ifndef QT_BOOTSTRAPPED
+# include <QtCore/qstandardpaths.h>
+#endif // QT_BOOTSTRAPPED
 
 #include <pwd.h>
 #include <stdlib.h> // for realpath()
@@ -55,7 +59,7 @@
 #include <stdio.h>
 #include <errno.h>
 
-#if QT_HAS_INCLUDE(<paths.h>)
+#if __has_include(<paths.h>)
 # include <paths.h>
 #endif
 #ifndef _PATH_TMP           // from <paths.h>
@@ -67,7 +71,7 @@
 # include <CoreFoundation/CFBundle.h>
 #endif
 
-#ifdef Q_OS_OSX
+#ifdef Q_OS_MACOS
 #include <CoreServices/CoreServices.h>
 #endif
 
@@ -190,7 +194,7 @@ namespace {
 namespace GetFileTimes {
 #if !QT_CONFIG(futimens) && (QT_CONFIG(futimes))
 template <typename T>
-static inline typename QtPrivate::QEnableIf<(&T::st_atim, &T::st_mtim, true)>::Type get(const T *p, struct timeval *access, struct timeval *modification)
+static inline typename std::enable_if_t<(&T::st_atim, &T::st_mtim, true)> get(const T *p, struct timeval *access, struct timeval *modification)
 {
     access->tv_sec = p->st_atim.tv_sec;
     access->tv_usec = p->st_atim.tv_nsec / 1000;
@@ -200,7 +204,7 @@ static inline typename QtPrivate::QEnableIf<(&T::st_atim, &T::st_mtim, true)>::T
 }
 
 template <typename T>
-static inline typename QtPrivate::QEnableIf<(&T::st_atimespec, &T::st_mtimespec, true)>::Type get(const T *p, struct timeval *access, struct timeval *modification)
+static inline typename std::enable_if_t<(&T::st_atimespec, &T::st_mtimespec, true)> get(const T *p, struct timeval *access, struct timeval *modification)
 {
     access->tv_sec = p->st_atimespec.tv_sec;
     access->tv_usec = p->st_atimespec.tv_nsec / 1000;
@@ -212,7 +216,7 @@ static inline typename QtPrivate::QEnableIf<(&T::st_atimespec, &T::st_mtimespec,
 #  ifndef st_atimensec
 // if "st_atimensec" is defined, this would expand to invalid C++
 template <typename T>
-static inline typename QtPrivate::QEnableIf<(&T::st_atimensec, &T::st_mtimensec, true)>::Type get(const T *p, struct timeval *access, struct timeval *modification)
+static inline typename std::enable_if_t<(&T::st_atimensec, &T::st_mtimensec, true)> get(const T *p, struct timeval *access, struct timeval *modification)
 {
     access->tv_sec = p->st_atime;
     access->tv_usec = p->st_atimensec / 1000;
@@ -229,51 +233,51 @@ qint64 timespecToMSecs(const timespec &spec)
 }
 
 // fallback set
-Q_DECL_UNUSED qint64 atime(const QT_STATBUF &statBuffer, ulong) { return qint64(statBuffer.st_atime) * 1000; }
-Q_DECL_UNUSED qint64 birthtime(const QT_STATBUF &, ulong)       { return Q_INT64_C(0); }
-Q_DECL_UNUSED qint64 ctime(const QT_STATBUF &statBuffer, ulong) { return qint64(statBuffer.st_ctime) * 1000; }
-Q_DECL_UNUSED qint64 mtime(const QT_STATBUF &statBuffer, ulong) { return qint64(statBuffer.st_mtime) * 1000; }
+[[maybe_unused]] qint64 atime(const QT_STATBUF &statBuffer, ulong) { return qint64(statBuffer.st_atime) * 1000; }
+[[maybe_unused]] qint64 birthtime(const QT_STATBUF &, ulong)       { return Q_INT64_C(0); }
+[[maybe_unused]] qint64 ctime(const QT_STATBUF &statBuffer, ulong) { return qint64(statBuffer.st_ctime) * 1000; }
+[[maybe_unused]] qint64 mtime(const QT_STATBUF &statBuffer, ulong) { return qint64(statBuffer.st_mtime) * 1000; }
 
 // Xtim, POSIX.1-2008
 template <typename T>
-Q_DECL_UNUSED static typename std::enable_if<(&T::st_atim, true), qint64>::type
+[[maybe_unused]] static typename std::enable_if<(&T::st_atim, true), qint64>::type
 atime(const T &statBuffer, int)
 { return timespecToMSecs(statBuffer.st_atim); }
 
 template <typename T>
-Q_DECL_UNUSED static typename std::enable_if<(&T::st_birthtim, true), qint64>::type
+[[maybe_unused]] static typename std::enable_if<(&T::st_birthtim, true), qint64>::type
 birthtime(const T &statBuffer, int)
 { return timespecToMSecs(statBuffer.st_birthtim); }
 
 template <typename T>
-Q_DECL_UNUSED static typename std::enable_if<(&T::st_ctim, true), qint64>::type
+[[maybe_unused]] static typename std::enable_if<(&T::st_ctim, true), qint64>::type
 ctime(const T &statBuffer, int)
 { return timespecToMSecs(statBuffer.st_ctim); }
 
 template <typename T>
-Q_DECL_UNUSED static typename std::enable_if<(&T::st_mtim, true), qint64>::type
+[[maybe_unused]] static typename std::enable_if<(&T::st_mtim, true), qint64>::type
 mtime(const T &statBuffer, int)
 { return timespecToMSecs(statBuffer.st_mtim); }
 
 #ifndef st_mtimespec
 // Xtimespec
 template <typename T>
-Q_DECL_UNUSED static typename std::enable_if<(&T::st_atimespec, true), qint64>::type
+[[maybe_unused]] static typename std::enable_if<(&T::st_atimespec, true), qint64>::type
 atime(const T &statBuffer, int)
 { return timespecToMSecs(statBuffer.st_atimespec); }
 
 template <typename T>
-Q_DECL_UNUSED static typename std::enable_if<(&T::st_birthtimespec, true), qint64>::type
+[[maybe_unused]] static typename std::enable_if<(&T::st_birthtimespec, true), qint64>::type
 birthtime(const T &statBuffer, int)
 { return timespecToMSecs(statBuffer.st_birthtimespec); }
 
 template <typename T>
-Q_DECL_UNUSED static typename std::enable_if<(&T::st_ctimespec, true), qint64>::type
+[[maybe_unused]] static typename std::enable_if<(&T::st_ctimespec, true), qint64>::type
 ctime(const T &statBuffer, int)
 { return timespecToMSecs(statBuffer.st_ctimespec); }
 
 template <typename T>
-Q_DECL_UNUSED static typename std::enable_if<(&T::st_mtimespec, true), qint64>::type
+[[maybe_unused]] static typename std::enable_if<(&T::st_mtimespec, true), qint64>::type
 mtime(const T &statBuffer, int)
 { return timespecToMSecs(statBuffer.st_mtimespec); }
 #endif
@@ -281,22 +285,22 @@ mtime(const T &statBuffer, int)
 #if !defined(st_mtimensec) && !defined(__alpha__)
 // Xtimensec
 template <typename T>
-Q_DECL_UNUSED static typename std::enable_if<(&T::st_atimensec, true), qint64>::type
+[[maybe_unused]] static typename std::enable_if<(&T::st_atimensec, true), qint64>::type
 atime(const T &statBuffer, int)
 { return statBuffer.st_atime * Q_INT64_C(1000) + statBuffer.st_atimensec / 1000000; }
 
 template <typename T>
-Q_DECL_UNUSED static typename std::enable_if<(&T::st_birthtimensec, true), qint64>::type
+[[maybe_unused]] static typename std::enable_if<(&T::st_birthtimensec, true), qint64>::type
 birthtime(const T &statBuffer, int)
 { return statBuffer.st_birthtime * Q_INT64_C(1000) + statBuffer.st_birthtimensec / 1000000; }
 
 template <typename T>
-Q_DECL_UNUSED static typename std::enable_if<(&T::st_ctimensec, true), qint64>::type
+[[maybe_unused]] static typename std::enable_if<(&T::st_ctimensec, true), qint64>::type
 ctime(const T &statBuffer, int)
 { return statBuffer.st_ctime * Q_INT64_C(1000) + statBuffer.st_ctimensec / 1000000; }
 
 template <typename T>
-Q_DECL_UNUSED static typename std::enable_if<(&T::st_mtimensec, true), qint64>::type
+[[maybe_unused]] static typename std::enable_if<(&T::st_mtimensec, true), qint64>::type
 mtime(const T &statBuffer, int)
 { return statBuffer.st_mtime * Q_INT64_C(1000) + statBuffer.st_mtimensec / 1000000; }
 #endif
@@ -500,8 +504,8 @@ void QFileSystemMetaData::fillFromStatBuf(const QT_STATBUF &statBuffer)
 void QFileSystemMetaData::fillFromDirEnt(const QT_DIRENT &entry)
 {
 #if defined(_DEXTRA_FIRST)
-    knownFlagsMask = 0;
-    entryFlags = 0;
+    knownFlagsMask = {};
+    entryFlags = {};
     for (dirent_extra *extra = _DEXTRA_FIRST(&entry); _DEXTRA_VALID(extra, &entry);
          extra = _DEXTRA_NEXT(extra)) {
         if (extra->d_type == _DTYPE_STAT || extra->d_type == _DTYPE_LSTAT) {
@@ -611,7 +615,7 @@ void QFileSystemMetaData::fillFromDirEnt(const QT_DIRENT &entry)
         clear();
     }
 #else
-    Q_UNUSED(entry)
+    Q_UNUSED(entry);
 #endif
 }
 
@@ -1076,14 +1080,14 @@ bool QFileSystemEngine::cloneFile(int srcfd, int dstfd, const QFileSystemMetaDat
     // sendfile(2) is limited in the kernel to 2G - 4k
     const size_t SendfileSize = 0x7ffff000;
 
-    ssize_t n = ::sendfile(dstfd, srcfd, NULL, SendfileSize);
+    ssize_t n = ::sendfile(dstfd, srcfd, nullptr, SendfileSize);
     if (n == -1) {
         // if we got an error here, give up and try at an upper layer
         return false;
     }
 
     while (n) {
-        n = ::sendfile(dstfd, srcfd, NULL, SendfileSize);
+        n = ::sendfile(dstfd, srcfd, nullptr, SendfileSize);
         if (n == -1) {
             // uh oh, this is probably a real error (like ENOSPC), but we have
             // no way to notify QFile of partial success, so just erase any work
@@ -1197,6 +1201,219 @@ bool QFileSystemEngine::createLink(const QFileSystemEntry &source, const QFileSy
     return false;
 }
 
+#ifndef Q_OS_DARWIN
+/*
+    Implementing as per https://specifications.freedesktop.org/trash-spec/trashspec-1.0.html
+*/
+
+// bootstrapped tools don't need this, and we don't want QStorageInfo
+#ifndef QT_BOOTSTRAPPED
+static QString freeDesktopTrashLocation(const QString &sourcePath)
+{
+    auto makeTrashDir = [](const QDir &topDir, const QString &trashDir) -> QString {
+        auto ownerPerms = QFileDevice::ReadOwner
+                        | QFileDevice::WriteOwner
+                        | QFileDevice::ExeOwner;
+        QString targetDir = topDir.filePath(trashDir);
+        // deliberately not using mkpath, since we want to fail if topDir doesn't exist
+        if (topDir.mkdir(trashDir))
+            QFile::setPermissions(targetDir, ownerPerms);
+        if (QFileInfo(targetDir).isDir())
+            return targetDir;
+        return QString();
+    };
+    auto isSticky = [](const QFileInfo &fileInfo) -> bool {
+        struct stat st;
+        if (stat(QFile::encodeName(fileInfo.absoluteFilePath()).constData(), &st) == 0)
+            return st.st_mode & S_ISVTX;
+
+        return false;
+    };
+
+    QString trash;
+    const QStorageInfo sourceStorage(sourcePath);
+    const QStorageInfo homeStorage(QDir::home());
+    // We support trashing of files outside the users home partition
+    if (sourceStorage != homeStorage) {
+        const QLatin1String dotTrash(".Trash");
+        QDir topDir(sourceStorage.rootPath());
+        /*
+            Method 1:
+            "An administrator can create an $topdir/.Trash directory. The permissions on this
+            directories should permit all users who can trash files at all to write in it;
+            and the “sticky bit” in the permissions must be set, if the file system supports
+            it.
+            When trashing a file from a non-home partition/device, an implementation
+            (if it supports trashing in top directories) MUST check for the presence
+            of $topdir/.Trash."
+        */
+        const QString userID = QString::number(::getuid());
+        if (topDir.cd(dotTrash)) {
+            const QFileInfo trashInfo(topDir.path());
+
+            // we MUST check that the sticky bit is set, and that it is not a symlink
+            if (trashInfo.isSymLink()) {
+                // we SHOULD report the failed check to the administrator
+                qCritical("Warning: '%s' is a symlink to '%s'",
+                          trashInfo.absoluteFilePath().toLocal8Bit().constData(),
+                          trashInfo.symLinkTarget().toLatin1().constData());
+            } else if (!isSticky(trashInfo)) {
+                // we SHOULD report the failed check to the administrator
+                qCritical("Warning: '%s' doesn't have sticky bit set!",
+                          trashInfo.absoluteFilePath().toLocal8Bit().constData());
+            } else if (trashInfo.isDir()) {
+                /*
+                    "If the directory exists and passes the checks, a subdirectory of the
+                     $topdir/.Trash directory is to be used as the user's trash directory
+                     for this partition/device. The name of this subdirectory is the numeric
+                     identifier of the current user ($topdir/.Trash/$uid).
+                     When trashing a file, if this directory does not exist for the current user,
+                     the implementation MUST immediately create it, without any warnings or
+                     delays for the user."
+                */
+                trash = makeTrashDir(topDir, userID);
+            }
+        }
+        /*
+            Method 2:
+            "If an $topdir/.Trash directory is absent, an $topdir/.Trash-$uid directory is to be
+             used as the user's trash directory for this device/partition. [...] When trashing a
+             file, if an $topdir/.Trash-$uid directory does not exist, the implementation MUST
+             immediately create it, without any warnings or delays for the user."
+        */
+        if (trash.isEmpty()) {
+            topDir = QDir(sourceStorage.rootPath());
+            const QString userTrashDir = dotTrash + QLatin1Char('-') + userID;
+            trash = makeTrashDir(topDir, userTrashDir);
+        }
+    }
+    /*
+        "If both (1) and (2) fail [...], the implementation MUST either trash the
+         file into the user's “home trash” or refuse to trash it."
+
+         We trash the file into the user's home trash.
+
+        "Its name and location are $XDG_DATA_HOME/Trash"; $XDG_DATA_HOME is what
+        QStandardPaths returns for GenericDataLocation. If that doesn't exist, then
+        we are not running on a freedesktop.org-compliant environment, and give up.
+    */
+    if (trash.isEmpty()) {
+        QDir topDir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation);
+        trash = makeTrashDir(topDir, QLatin1String("Trash"));
+        if (!QFileInfo(trash).isDir()) {
+            qWarning("Unable to establish trash directory in %s",
+                     topDir.path().toLocal8Bit().constData());
+        }
+    }
+
+    return trash;
+}
+#endif // QT_BOOTSTRAPPED
+
+//static
+bool QFileSystemEngine::moveFileToTrash(const QFileSystemEntry &source,
+                                        QFileSystemEntry &newLocation, QSystemError &error)
+{
+#ifdef QT_BOOTSTRAPPED
+    Q_UNUSED(source);
+    Q_UNUSED(newLocation);
+    error = QSystemError(ENOSYS, QSystemError::StandardLibraryError);
+    return false;
+#else
+    const QFileInfo sourceInfo(source.filePath());
+    if (!sourceInfo.exists()) {
+        error = QSystemError(ENOENT, QSystemError::StandardLibraryError);
+        return false;
+    }
+    const QString sourcePath = sourceInfo.absoluteFilePath();
+
+    QDir trashDir(freeDesktopTrashLocation(sourcePath));
+    if (!trashDir.exists())
+        return false;
+    /*
+        "A trash directory contains two subdirectories, named info and files."
+    */
+    const QLatin1String filesDir("files");
+    const QLatin1String infoDir("info");
+    trashDir.mkdir(filesDir);
+    int savedErrno = errno;
+    trashDir.mkdir(infoDir);
+    if (!savedErrno)
+        savedErrno = errno;
+    if (!trashDir.exists(filesDir) || !trashDir.exists(infoDir)) {
+        error = QSystemError(savedErrno, QSystemError::StandardLibraryError);
+        return false;
+    }
+    /*
+        "The $trash/files directory contains the files and directories that were trashed.
+         The names of files in this directory are to be determined by the implementation;
+         the only limitation is that they must be unique within the directory. Even if a
+         file with the same name and location gets trashed many times, each subsequent
+         trashing must not overwrite a previous copy."
+    */
+    const QString trashedName = sourceInfo.isDir()
+                              ? QDir(sourcePath).dirName()
+                              : sourceInfo.fileName();
+    QString uniqueTrashedName = QLatin1Char('/') + trashedName;
+    QString infoFileName;
+    int counter = 0;
+    QFile infoFile;
+    auto makeUniqueTrashedName = [trashedName, &counter]() -> QString {
+        ++counter;
+        return QString(QLatin1String("/%1-%2"))
+                                        .arg(trashedName)
+                                        .arg(counter, 4, 10, QLatin1Char('0'));
+    };
+    do {
+        while (QFile::exists(trashDir.filePath(filesDir) + uniqueTrashedName))
+            uniqueTrashedName = makeUniqueTrashedName();
+        /*
+            "The $trash/info directory contains an "information file" for every file and directory
+             in $trash/files. This file MUST have exactly the same name as the file or directory in
+             $trash/files, plus the extension ".trashinfo"
+             [...]
+             When trashing a file or directory, the implementation MUST create the corresponding
+             file in $trash/info first. Moreover, it MUST try to do this in an atomic fashion,
+             so that if two processes try to trash files with the same filename this will result
+             in two different trash files. On Unix-like systems this is done by generating a
+             filename, and then opening with O_EXCL. If that succeeds the creation was atomic
+             (at least on the same machine), if it fails you need to pick another filename."
+        */
+        infoFileName = trashDir.filePath(infoDir)
+                     + uniqueTrashedName + QLatin1String(".trashinfo");
+        infoFile.setFileName(infoFileName);
+        if (!infoFile.open(QIODevice::NewOnly | QIODevice::WriteOnly | QIODevice::Text))
+            uniqueTrashedName = makeUniqueTrashedName();
+    } while (!infoFile.isOpen());
+
+    const QString targetPath = trashDir.filePath(filesDir) + uniqueTrashedName;
+    const QFileSystemEntry target(targetPath);
+
+    /*
+        We might fail to rename if source and target are on different file systems.
+        In that case, we don't try further, i.e. copying and removing the original
+        is usually not what the user would expect to happen.
+    */
+    if (!renameFile(source, target, error)) {
+        infoFile.close();
+        infoFile.remove();
+        return false;
+    }
+
+    QByteArray info =
+            "[Trash Info]\n"
+            "Path=" + sourcePath.toUtf8() + "\n"
+            "DeletionDate=" + QDateTime::currentDateTime().toString(QLatin1String("yyyy-MM-ddThh:mm:ss")).toUtf8()
+            + "\n";
+    infoFile.write(info);
+    infoFile.close();
+
+    newLocation = QFileSystemEntry(targetPath);
+    return true;
+#endif // QT_BOOTSTRAPPED
+}
+#endif // Q_OS_DARWIN
+
 //static
 bool QFileSystemEngine::copyFile(const QFileSystemEntry &source, const QFileSystemEntry &target, QSystemError &error)
 {
