@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2020 The Qt Company Ltd.
 ** Copyright (C) 2016 Intel Corporation.
 ** Contact: https://www.qt.io/licensing/
 **
@@ -51,32 +51,11 @@
 #   include <time.h>
 #endif
 
-#ifdef Q_OS_WINRT
-#include <qfunctions_winrt.h>
-
-#include <wrl.h>
-#include <windows.foundation.h>
-#include <windows.foundation.collections.h>
-#include <windows.system.userprofile.h>
-#endif // Q_OS_WINRT
-
 QT_BEGIN_NAMESPACE
 
-#ifndef Q_OS_WINRT
 static QByteArray getWinLocaleName(LCID id = LOCALE_USER_DEFAULT);
 static QString winIso639LangName(LCID id = LOCALE_USER_DEFAULT);
 static QString winIso3116CtryName(LCID id = LOCALE_USER_DEFAULT);
-#else // !Q_OS_WINRT
-using namespace Microsoft::WRL;
-using namespace Microsoft::WRL::Wrappers;
-using namespace ABI::Windows::Foundation;
-using namespace ABI::Windows::System::UserProfile;
-
-static QByteArray getWinLocaleName(LPWSTR id = LOCALE_NAME_USER_DEFAULT);
-static const char *winLangCodeToIsoName(int code);
-static QString winIso639LangName(LPWSTR id = LOCALE_NAME_USER_DEFAULT);
-static QString winIso3116CtryName(LPWSTR id = LOCALE_NAME_USER_DEFAULT);
-#endif // Q_OS_WINRT
 
 #ifndef QT_NO_SYSTEMLOCALE
 
@@ -106,11 +85,11 @@ struct QSystemLocalePrivate
 {
     QSystemLocalePrivate();
 
-    QChar zeroDigit();
-    QChar decimalPoint();
-    QChar groupSeparator();
-    QChar negativeSign();
-    QChar positiveSign();
+    QVariant zeroDigit();
+    QVariant decimalPoint();
+    QVariant groupSeparator();
+    QVariant negativeSign();
+    QVariant positiveSign();
     QVariant dateFormat(QLocale::FormatType);
     QVariant timeFormat(QLocale::FormatType);
     QVariant dateTimeFormat(QLocale::FormatType);
@@ -141,25 +120,20 @@ private:
     };
 
     // cached values:
-#ifndef Q_OS_WINRT
     LCID lcid;
-#else
-    WCHAR lcName[LOCALE_NAME_MAX_LENGTH];
-#endif
     SubstitutionType substitutionType;
-    QChar zero;
+    QString zero; // cached value for zeroDigit()
 
     int getLocaleInfo(LCTYPE type, LPWSTR data, int size);
-    QString getLocaleInfo(LCTYPE type, int maxlen = 0);
-    int getLocaleInfo_int(LCTYPE type, int maxlen = 0);
-    QChar getLocaleInfo_qchar(LCTYPE type);
+    QVariant getLocaleInfo(LCTYPE type);
+    int getLocaleInfo_int(LCTYPE type);
 
     int getCurrencyFormat(DWORD flags, LPCWSTR value, const CURRENCYFMTW *format, LPWSTR data, int size);
     int getDateFormat(DWORD flags, const SYSTEMTIME * date, LPCWSTR format, LPWSTR data, int size);
     int getTimeFormat(DWORD flags, const SYSTEMTIME *date, LPCWSTR format, LPWSTR data, int size);
 
     SubstitutionType substitution();
-    QString &substituteDigits(QString &string);
+    QString substituteDigits(QString &&string);
 
     static QString winToQtFormat(QStringView sys_fmt);
 
@@ -169,79 +143,64 @@ Q_GLOBAL_STATIC(QSystemLocalePrivate, systemLocalePrivate)
 QSystemLocalePrivate::QSystemLocalePrivate()
     : substitutionType(SUnknown)
 {
-#ifndef Q_OS_WINRT
     lcid = GetUserDefaultLCID();
-#else
-    GetUserDefaultLocaleName(lcName, LOCALE_NAME_MAX_LENGTH);
-#endif
 }
 
 inline int QSystemLocalePrivate::getCurrencyFormat(DWORD flags, LPCWSTR value, const CURRENCYFMTW *format, LPWSTR data, int size)
 {
-#ifndef Q_OS_WINRT
     return GetCurrencyFormat(lcid, flags, value, format, data, size);
-#else
-    return GetCurrencyFormatEx(lcName, flags, value, format, data, size);
-#endif
 }
 
 inline int QSystemLocalePrivate::getDateFormat(DWORD flags, const SYSTEMTIME * date, LPCWSTR format, LPWSTR data, int size)
 {
-#ifndef Q_OS_WINRT
     return GetDateFormat(lcid, flags, date, format, data, size);
-#else
-    return GetDateFormatEx(lcName, flags, date, format, data, size, NULL);
-#endif
 }
 
 inline int QSystemLocalePrivate::getTimeFormat(DWORD flags, const SYSTEMTIME *date, LPCWSTR format, LPWSTR data, int size)
 {
-#ifndef Q_OS_WINRT
     return GetTimeFormat(lcid, flags, date, format, data, size);
-#else
-    return GetTimeFormatEx(lcName, flags, date, format, data, size);
-#endif
 }
 
 inline int QSystemLocalePrivate::getLocaleInfo(LCTYPE type, LPWSTR data, int size)
 {
-#ifndef Q_OS_WINRT
     return GetLocaleInfo(lcid, type, data, size);
-#else
-    return GetLocaleInfoEx(lcName, type, data, size);
-#endif
 }
 
-QString QSystemLocalePrivate::getLocaleInfo(LCTYPE type, int maxlen)
+QVariant QSystemLocalePrivate::getLocaleInfo(LCTYPE type)
 {
-    QVarLengthArray<wchar_t, 64> buf(maxlen ? maxlen : 64);
-    if (!getLocaleInfo(type, buf.data(), buf.size()))
-        return QString();
-    if (GetLastError() == ERROR_INSUFFICIENT_BUFFER) {
+    // https://docs.microsoft.com/en-us/windows/win32/intl/locale-spositivesign
+    // says empty for LOCALE_SPOSITIVESIGN means "+", although GetLocaleInfo()
+    // is documented to return 0 only on failure, so it's not clear how it
+    // returns empty to mean this; hence the two checks for it below.
+    const QString plus = QStringLiteral("+");
+    QVarLengthArray<wchar_t, 64> buf(64);
+    // Need to distinguish empty QString packaged as (non-null) QVariant from null QVariant:
+    if (!getLocaleInfo(type, buf.data(), buf.size())) {
+        const auto lastError = GetLastError();
+        if (type == LOCALE_SPOSITIVESIGN && lastError == ERROR_SUCCESS)
+            return plus;
+        if (lastError != ERROR_INSUFFICIENT_BUFFER)
+            return {};
         int cnt = getLocaleInfo(type, 0, 0);
         if (cnt == 0)
-            return QString();
+            return {};
         buf.resize(cnt);
         if (!getLocaleInfo(type, buf.data(), buf.size()))
-            return QString();
+            return {};
     }
+    if (type == LOCALE_SPOSITIVESIGN && !buf[0])
+        return plus;
     return QString::fromWCharArray(buf.data());
 }
 
-int QSystemLocalePrivate::getLocaleInfo_int(LCTYPE type, int maxlen)
+int QSystemLocalePrivate::getLocaleInfo_int(LCTYPE type)
 {
-    QString str = getLocaleInfo(type, maxlen);
+    const QString str = getLocaleInfo(type).toString();
     bool ok = false;
-    int v = str.toInt(&ok);
+    const int v = str.toInt(&ok);
     return ok ? v : 0;
 }
 
-QChar QSystemLocalePrivate::getLocaleInfo_qchar(LCTYPE type)
-{
-    QString str = getLocaleInfo(type);
-    return str.isEmpty() ? QChar() : str.at(0);
-}
-
 QSystemLocalePrivate::SubstitutionType QSystemLocalePrivate::substitution()
 {
     if (substitutionType == SUnknown) {
@@ -257,13 +216,12 @@ QSystemLocalePrivate::SubstitutionType QSystemLocalePrivate::substitution()
         else if (buf[0] == '2')
             substitutionType = QSystemLocalePrivate::SAlways;
         else {
-            wchar_t digits[11];
+            wchar_t digits[11]; // See zeroDigit() for why 11.
             if (!getLocaleInfo(LOCALE_SNATIVEDIGITS, digits, 11)) {
                 substitutionType = QSystemLocalePrivate::SNever;
                 return substitutionType;
             }
-            const wchar_t zero = digits[0];
-            if (buf[0] == zero + 2)
+            if (buf[0] == digits[0] + 2)
                 substitutionType = QSystemLocalePrivate::SAlways;
             else
                 substitutionType = QSystemLocalePrivate::SNever;
@@ -272,51 +230,88 @@ QSystemLocalePrivate::SubstitutionType QSystemLocalePrivate::substitution()
     return substitutionType;
 }
 
-QString &QSystemLocalePrivate::substituteDigits(QString &string)
+QString QSystemLocalePrivate::substituteDigits(QString &&string)
 {
-    ushort zero = zeroDigit().unicode();
-    ushort *qch = reinterpret_cast<ushort *>(string.data());
-    for (ushort *end = qch + string.size(); qch != end; ++qch) {
-        if (*qch >= '0' && *qch <= '9')
-            *qch = zero + (*qch - '0');
+    zeroDigit(); // Ensure zero is set.
+    switch (zero.size()) {
+    case 1: {
+        ushort z = zero.at(0).unicode();
+        if (z == '0') // Nothing to do
+            break;
+        Q_ASSERT(z > '9');
+        ushort *const qch = reinterpret_cast<ushort *>(string.data());
+        for (int i = 0, stop = string.size(); i < stop; ++i) {
+            ushort &ch = qch[i];
+            if (ch >= '0' && ch <= '9')
+                ch = unicodeForDigit(ch - '0', z);
+        }
+        break;
     }
-    return string;
+    case 2: {
+        // Surrogate pair (high, low):
+        char32_t z = QChar::surrogateToUcs4(zero.at(0), zero.at(1));
+        for (int i = 0; i < 10; i++) {
+            char32_t digit = unicodeForDigit(i, z);
+            const QChar s[2] = { QChar::highSurrogate(digit), QChar::lowSurrogate(digit) };
+            string.replace(QString(QLatin1Char('0' + i)), QString(s, 2));
+        }
+        break;
+    }
+    default:
+        Q_ASSERT(!"Expected zero digit to be a single UCS2 code-point or a surrogate pair");
+    case 0: // Apparently this locale info was not available.
+        break;
+    }
+    return std::move(string);
 }
 
-QChar QSystemLocalePrivate::zeroDigit()
+QVariant QSystemLocalePrivate::zeroDigit()
 {
-    if (zero.isNull())
-        zero = getLocaleInfo_qchar(LOCALE_SNATIVEDIGITS);
+    if (zero.isEmpty()) {
+        /* Ten digits plus a terminator.
+
+           https://docs.microsoft.com/en-us/windows/win32/intl/locale-snative-constants
+           "Native equivalents of ASCII 0 through 9. The maximum number of
+           characters allowed for this string is eleven, including a terminating
+           null character."
+         */
+        wchar_t digits[11];
+        if (getLocaleInfo(LOCALE_SNATIVEDIGITS, digits, 11)) {
+            // assert all(digits[i] == i + digits[0] for i in range(1, 10)),
+            // assumed above (unless digits[0] is 0x3007; see QTBUG-85409).
+            zero = QString::fromWCharArray(digits, 1);
+        }
+    }
     return zero;
 }
 
-QChar QSystemLocalePrivate::decimalPoint()
+QVariant QSystemLocalePrivate::decimalPoint()
 {
-    return getLocaleInfo_qchar(LOCALE_SDECIMAL);
+    return getLocaleInfo(LOCALE_SDECIMAL);
 }
 
-QChar QSystemLocalePrivate::groupSeparator()
+QVariant QSystemLocalePrivate::groupSeparator()
 {
-    return getLocaleInfo_qchar(LOCALE_STHOUSAND);
+    return getLocaleInfo(LOCALE_STHOUSAND);
 }
 
-QChar QSystemLocalePrivate::negativeSign()
+QVariant QSystemLocalePrivate::negativeSign()
 {
-    return getLocaleInfo_qchar(LOCALE_SNEGATIVESIGN);
+    return getLocaleInfo(LOCALE_SNEGATIVESIGN);
 }
 
-QChar QSystemLocalePrivate::positiveSign()
+QVariant QSystemLocalePrivate::positiveSign()
 {
-    return getLocaleInfo_qchar(LOCALE_SPOSITIVESIGN);
+    return getLocaleInfo(LOCALE_SPOSITIVESIGN);
 }
 
 QVariant QSystemLocalePrivate::dateFormat(QLocale::FormatType type)
 {
     switch (type) {
     case QLocale::ShortFormat:
-        return winToQtFormat(getLocaleInfo(LOCALE_SSHORTDATE));
+        return winToQtFormat(getLocaleInfo(LOCALE_SSHORTDATE).toString());
     case QLocale::LongFormat:
-        return winToQtFormat(getLocaleInfo(LOCALE_SLONGDATE));
+        return winToQtFormat(getLocaleInfo(LOCALE_SLONGDATE).toString());
     case QLocale::NarrowFormat:
         break;
     }
@@ -327,9 +322,9 @@ QVariant QSystemLocalePrivate::timeFormat(QLocale::FormatType type)
 {
     switch (type) {
     case QLocale::ShortFormat:
-        return winToQtFormat(getLocaleInfo(LOCALE_SSHORTTIME));
+        return winToQtFormat(getLocaleInfo(LOCALE_SSHORTTIME).toString());
     case QLocale::LongFormat:
-        return winToQtFormat(getLocaleInfo(LOCALE_STIMEFORMAT));
+        return winToQtFormat(getLocaleInfo(LOCALE_STIMEFORMAT).toString());
     case QLocale::NarrowFormat:
         break;
     }
@@ -407,7 +402,7 @@ QVariant QSystemLocalePrivate::toString(QDate date, QLocale::FormatType type)
     if (getDateFormat(flags, &st, NULL, buf, 255)) {
         QString format = QString::fromWCharArray(buf);
         if (substitution() == SAlways)
-            substituteDigits(format);
+            format = substituteDigits(std::move(format));
         return format;
     }
     return QString();
@@ -431,7 +426,7 @@ QVariant QSystemLocalePrivate::toString(QTime time, QLocale::FormatType type)
     if (getTimeFormat(flags, &st, NULL, buf, 255)) {
         QString format = QString::fromWCharArray(buf);
         if (substitution() == SAlways)
-            substituteDigits(format);
+            format = substituteDigits(std::move(format));
         return format;
     }
     return QString();
@@ -465,9 +460,8 @@ QVariant QSystemLocalePrivate::amText()
 {
     wchar_t output[15]; // maximum length including  terminating zero character for Win2003+
 
-    if (getLocaleInfo(LOCALE_S1159, output, 15)) {
+    if (getLocaleInfo(LOCALE_S1159, output, 15))
         return QString::fromWCharArray(output);
-    }
 
     return QVariant();
 }
@@ -476,9 +470,8 @@ QVariant QSystemLocalePrivate::pmText()
 {
     wchar_t output[15]; // maximum length including  terminating zero character for Win2003+
 
-    if (getLocaleInfo(LOCALE_S2359, output, 15)) {
+    if (getLocaleInfo(LOCALE_S2359, output, 15))
         return QString::fromWCharArray(output);
-    }
 
     return QVariant();
 }
@@ -525,27 +518,26 @@ QVariant QSystemLocalePrivate::currencySymbol(QLocale::CurrencySymbolFormat form
 QVariant QSystemLocalePrivate::toCurrencyString(const QSystemLocale::CurrencyToStringArgument &arg)
 {
     QString value;
-    switch (arg.value.type()) {
-    case QVariant::Int:
-        value = QLocaleData::longLongToString(QLatin1Char('0'), QLatin1Char(','), QLatin1Char('+'), QLatin1Char('-'),
-                                                 arg.value.toInt(), -1, 10, -1, QLocale::OmitGroupSeparator);
+    switch (arg.value.typeId()) {
+    case QMetaType::Int:
+        value = QLocaleData::c()->longLongToString(
+            arg.value.toInt(), -1, 10, -1, QLocale::OmitGroupSeparator);
         break;
-    case QVariant::UInt:
-        value = QLocaleData::unsLongLongToString(QLatin1Char('0'), QLatin1Char(','), QLatin1Char('+'),
-                                                    arg.value.toUInt(), -1, 10, -1, QLocale::OmitGroupSeparator);
+    case QMetaType::UInt:
+        value = QLocaleData::c()->unsLongLongToString(
+            arg.value.toUInt(), -1, 10, -1, QLocale::OmitGroupSeparator);
         break;
-    case QVariant::Double:
-        value = QLocaleData::doubleToString(QLatin1Char('0'), QLatin1Char('+'), QLatin1Char('-'),
-                                               QLatin1Char(' '), QLatin1Char(','), QLatin1Char('.'),
-                                               arg.value.toDouble(), -1, QLocaleData::DFDecimal, -1, QLocale::OmitGroupSeparator);
+    case QMetaType::Double:
+        value = QLocaleData::c()->doubleToString(
+            arg.value.toDouble(), -1, QLocaleData::DFDecimal, -1, QLocale::OmitGroupSeparator);
         break;
-    case QVariant::LongLong:
-        value = QLocaleData::longLongToString(QLatin1Char('0'), QLatin1Char(','), QLatin1Char('+'), QLatin1Char('-'),
-                                                 arg.value.toLongLong(), -1, 10, -1, QLocale::OmitGroupSeparator);
+    case QMetaType::LongLong:
+        value = QLocaleData::c()->longLongToString(
+            arg.value.toLongLong(), -1, 10, -1, QLocale::OmitGroupSeparator);
         break;
-    case QVariant::ULongLong:
-        value = QLocaleData::unsLongLongToString(QLatin1Char('0'), QLatin1Char(','), QLatin1Char('+'),
-                                                    arg.value.toULongLong(), -1, 10, -1, QLocale::OmitGroupSeparator);
+    case QMetaType::ULongLong:
+        value = QLocaleData::c()->unsLongLongToString(
+            arg.value.toULongLong(), -1, 10, -1, QLocale::OmitGroupSeparator);
         break;
     default:
         return QVariant();
@@ -560,9 +552,9 @@ QVariant QSystemLocalePrivate::toCurrencyString(const QSystemLocale::CurrencyToS
     if (!arg.symbol.isEmpty()) {
         format.NumDigits = getLocaleInfo_int(LOCALE_ICURRDIGITS);
         format.LeadingZero = getLocaleInfo_int(LOCALE_ILZERO);
-        decimalSep = getLocaleInfo(LOCALE_SMONDECIMALSEP);
+        decimalSep = getLocaleInfo(LOCALE_SMONDECIMALSEP).toString();
         format.lpDecimalSep = (wchar_t *)decimalSep.utf16();
-        thousandSep = getLocaleInfo(LOCALE_SMONTHOUSANDSEP);
+        thousandSep = getLocaleInfo(LOCALE_SMONTHOUSANDSEP).toString();
         format.lpThousandSep = (wchar_t *)thousandSep.utf16();
         format.NegativeOrder = getLocaleInfo_int(LOCALE_INEGCURR);
         format.PositiveOrder = getLocaleInfo_int(LOCALE_ICURRENCY);
@@ -574,7 +566,7 @@ QVariant QSystemLocalePrivate::toCurrencyString(const QSystemLocale::CurrencyToS
         // int(30) == "123456,789.00"   == string("3;0;0")
         // int(32) == "12,34,56,789.00" == string("3;2;0")
         // int(320)== "1234,56,789.00"  == string("3;2")
-        QString groupingStr = getLocaleInfo(LOCALE_SMONGROUPING);
+        QString groupingStr = getLocaleInfo(LOCALE_SMONGROUPING).toString();
         format.Grouping = groupingStr.remove(QLatin1Char(';')).toInt();
         if (format.Grouping % 10 == 0) // magic
             format.Grouping /= 10;
@@ -595,13 +587,12 @@ QVariant QSystemLocalePrivate::toCurrencyString(const QSystemLocale::CurrencyToS
 
     value = QString::fromWCharArray(out.data());
     if (substitution() == SAlways)
-        substituteDigits( value);
+        value = substituteDigits(std::move(value));
     return value;
 }
 
 QVariant QSystemLocalePrivate::uiLanguages()
 {
-#ifndef Q_OS_WINRT
     unsigned long cnt = 0;
     QVarLengthArray<wchar_t, 64> buf(64);
 #  if !defined(QT_BOOTSTRAPPED) && !defined(QT_BUILD_QMAKE) // Not present in MinGW 4.9/bootstrap builds.
@@ -627,35 +618,6 @@ QVariant QSystemLocalePrivate::uiLanguages()
         str += s.size() + 1;
     }
     return result;
-#else // !Q_OS_WINRT
-    QStringList result;
-
-    ComPtr<IGlobalizationPreferencesStatics> preferences;
-    HRESULT hr = GetActivationFactory(HString::MakeReference(RuntimeClass_Windows_System_UserProfile_GlobalizationPreferences).Get(), &preferences);
-    if (FAILED(hr)) {
-        qWarning("Could not obtain ApplicationLanguagesStatic");
-        return QStringList();
-    }
-
-    ComPtr<ABI::Windows::Foundation::Collections::IVectorView<HSTRING> > languageList;
-    // Languages is a ranked list of "long names" (e.g. en-US) of preferred languages
-    hr = preferences->get_Languages(&languageList);
-    Q_ASSERT_SUCCEEDED(hr);
-    unsigned int size;
-    hr = languageList->get_Size(&size);
-    Q_ASSERT_SUCCEEDED(hr);
-    result.reserve(size);
-    for (unsigned int i = 0; i < size; ++i) {
-        HString language;
-        hr = languageList->GetAt(i, language.GetAddressOf());
-        Q_ASSERT_SUCCEEDED(hr);
-        UINT32 length;
-        PCWSTR rawString = language.GetRawBuffer(&length);
-        result << QString::fromWCharArray(rawString, length);
-    }
-
-    return result;
-#endif // Q_OS_WINRT
 }
 
 QVariant QSystemLocalePrivate::nativeLanguageName()
@@ -671,13 +633,9 @@ QVariant QSystemLocalePrivate::nativeCountryName()
 
 void QSystemLocalePrivate::update()
 {
-#ifndef Q_OS_WINRT
     lcid = GetUserDefaultLCID();
-#else
-    GetUserDefaultLocaleName(lcName, LOCALE_NAME_MAX_LENGTH);
-#endif
     substitutionType = SUnknown;
-    zero = QChar();
+    zero.resize(0);
 }
 
 QString QSystemLocalePrivate::winToQtFormat(QStringView sys_fmt)
@@ -749,7 +707,7 @@ QLocale QSystemLocale::fallbackUiLocale() const
     return QLocale(QString::fromLatin1(getWinLocaleName()));
 }
 
-QVariant QSystemLocale::query(QueryType type, QVariant in = QVariant()) const
+QVariant QSystemLocale::query(QueryType type, QVariant in) const
 {
     QSystemLocalePrivate *d = systemLocalePrivate();
     switch(type) {
@@ -798,20 +756,15 @@ QVariant QSystemLocale::query(QueryType type, QVariant in = QVariant()) const
     case ZeroDigit:
         return d->zeroDigit();
     case LanguageId:
-    case CountryId: {
-        QString locale = QString::fromLatin1(getWinLocaleName());
-        QLocale::Language lang;
-        QLocale::Script script;
-        QLocale::Country cntry;
-        QLocalePrivate::getLangAndCountry(locale, lang, script, cntry);
-        if (type == LanguageId)
-            return lang;
-        if (cntry == QLocale::AnyCountry)
-            return fallbackUiLocale().country();
-        return cntry;
-    }
     case ScriptId:
-        return QVariant(QLocale::AnyScript);
+    case CountryId: {
+        QLocaleId lid = QLocaleId::fromName(QString::fromLatin1(getWinLocaleName()));
+        if (type == LanguageId)
+            return lid.language_id;
+        if (type == ScriptId)
+            return lid.script_id ? lid.script_id : ushort(fallbackUiLocale().script());
+        return lid.country_id ? lid.country_id : ushort(fallbackUiLocale().country());
+    }
     case MeasurementSystem:
         return d->measurementSystem();
     case Collation:
@@ -1017,11 +970,7 @@ LCID qt_inIsoNametoLCID(const char *name)
 }
 
 
-#ifndef Q_OS_WINRT
 static QString winIso639LangName(LCID id)
-#else
-static QString winIso639LangName(LPWSTR id)
-#endif
 {
     QString result;
 
@@ -1029,11 +978,7 @@ static QString winIso639LangName(LPWSTR id)
     // the language code
     QString lang_code;
     wchar_t out[256];
-#ifndef Q_OS_WINRT
     if (GetLocaleInfo(id, LOCALE_ILANGUAGE, out, 255))
-#else
-    if (GetLocaleInfoEx(id, LOCALE_ILANGUAGE, out, 255))
-#endif
         lang_code = QString::fromWCharArray(out);
 
     if (!lang_code.isEmpty()) {
@@ -1056,71 +1001,41 @@ static QString winIso639LangName(LPWSTR id)
         return result;
 
     // not one of the problematic languages - do the usual lookup
-#ifndef Q_OS_WINRT
     if (GetLocaleInfo(id, LOCALE_SISO639LANGNAME, out, 255))
-#else
-    if (GetLocaleInfoEx(id, LOCALE_SISO639LANGNAME, out, 255))
-#endif
         result = QString::fromWCharArray(out);
 
     return result;
 }
 
-#ifndef Q_OS_WINRT
 static QString winIso3116CtryName(LCID id)
-#else
-static QString winIso3116CtryName(LPWSTR id)
-#endif
 {
     QString result;
 
     wchar_t out[256];
-#ifndef Q_OS_WINRT
     if (GetLocaleInfo(id, LOCALE_SISO3166CTRYNAME, out, 255))
-#else
-    if (GetLocaleInfoEx(id, LOCALE_SISO3166CTRYNAME, out, 255))
-#endif
         result = QString::fromWCharArray(out);
 
     return result;
 }
 
-#ifndef Q_OS_WINRT
 static QByteArray getWinLocaleName(LCID id)
-#else
-static QByteArray getWinLocaleName(LPWSTR id)
-#endif
 {
     QByteArray result;
-#ifndef Q_OS_WINRT
     if (id == LOCALE_USER_DEFAULT) {
-#else
-    if (QString::fromWCharArray(id) == QString::fromWCharArray(LOCALE_NAME_USER_DEFAULT)) {
-#endif
-        static QByteArray langEnvVar = qgetenv("LANG");
+        static const QByteArray langEnvVar = qgetenv("LANG");
         result = langEnvVar;
-        QString lang, script, cntry;
-        if ( result == "C" || (!result.isEmpty()
-                && qt_splitLocaleName(QString::fromLocal8Bit(result), lang, script, cntry)) ) {
-            long id = 0;
-            bool ok = false;
-            id = qstrtoll(result.data(), 0, 0, &ok);
-            if ( !ok || id == 0 || id < INT_MIN || id > INT_MAX )
+        if (result == "C"
+            || (!result.isEmpty() && qt_splitLocaleName(QString::fromLocal8Bit(result)))) {
+            bool ok = false; // See if we have a Windows locale code instead of a locale name:
+            long id = qstrtoll(result.data(), 0, 0, &ok);
+            if (!ok || id == 0 || id < INT_MIN || id > INT_MAX) // Assume real locale name
                 return result;
             return winLangCodeToIsoName(int(id));
         }
     }
 
-#ifndef Q_OS_WINRT
     if (id == LOCALE_USER_DEFAULT)
         id = GetUserDefaultLCID();
-#else // !Q_OS_WINRT
-    WCHAR lcName[LOCALE_NAME_MAX_LENGTH];
-    if (QString::fromWCharArray(id) == QString::fromWCharArray(LOCALE_NAME_USER_DEFAULT)) {
-        GetUserDefaultLocaleName(lcName, LOCALE_NAME_MAX_LENGTH);
-        id = lcName;
-    }
-#endif // Q_OS_WINRT
     QString resultusage = winIso639LangName(id);
     QString country = winIso3116CtryName(id);
     if (!country.isEmpty())
@@ -1131,13 +1046,7 @@ static QByteArray getWinLocaleName(LPWSTR id)
 
 Q_CORE_EXPORT QLocale qt_localeFromLCID(LCID id)
 {
-#ifndef Q_OS_WINRT
     return QLocale(QString::fromLatin1(getWinLocaleName(id)));
-#else // !Q_OS_WINRT
-    WCHAR name[LOCALE_NAME_MAX_LENGTH];
-    LCIDToLocaleName(id, name, LOCALE_NAME_MAX_LENGTH, 0);
-    return QLocale(QString::fromLatin1(getWinLocaleName(name)));
-#endif // Q_OS_WINRT
 }
 
 QT_END_NAMESPACE
