@@ -67,25 +67,52 @@ QT_BEGIN_NAMESPACE
     desktop environment's settings panel, to let the user globally control UI
     and font sizes in different applications.
 
+    \note Both physical and logical DPI are expressed in device-independent dots.
+    Multiply by QScreen::devicePixelRatio() to get device-dependent density.
+
     \inmodule QtGui
 */
 
 QScreen::QScreen(QPlatformScreen *screen)
-    : QObject(*new QScreenPrivate(), 0)
+    : QObject(*new QScreenPrivate(), nullptr)
 {
     Q_D(QScreen);
     d->setPlatformScreen(screen);
 }
 
+void QScreenPrivate::updateGeometriesWithSignals()
+{
+    const QRect oldGeometry = geometry;
+    const QRect oldAvailableGeometry = availableGeometry;
+    updateHighDpi();
+    emitGeometryChangeSignals(oldGeometry != geometry, oldAvailableGeometry != availableGeometry);
+}
+
+void QScreenPrivate::emitGeometryChangeSignals(bool geometryChanged, bool availableGeometryChanged)
+{
+    Q_Q(QScreen);
+    if (geometryChanged)
+        emit q->geometryChanged(geometry);
+
+    if (availableGeometryChanged)
+        emit q->availableGeometryChanged(availableGeometry);
+
+    if (geometryChanged || availableGeometryChanged) {
+        const auto siblings = q->virtualSiblings();
+        for (QScreen* sibling : siblings)
+            emit sibling->virtualGeometryChanged(sibling->virtualGeometry());
+    }
+
+    if (geometryChanged)
+        emit q->physicalDotsPerInchChanged(q->physicalDotsPerInch());
+}
+
 void QScreenPrivate::setPlatformScreen(QPlatformScreen *screen)
 {
     Q_Q(QScreen);
     platformScreen = screen;
     platformScreen->d_func()->screen = q;
     orientation = platformScreen->orientation();
-    geometry = platformScreen->deviceIndependentGeometry();
-    availableGeometry = QHighDpi::fromNative(platformScreen->availableGeometry(),
-                        QHighDpiScaling::factor(platformScreen), geometry.topLeft());
 
     logicalDpi = QPlatformScreen::overrideDpi(platformScreen->logicalDpi());
 
@@ -94,13 +121,8 @@ void QScreenPrivate::setPlatformScreen(QPlatformScreen *screen)
     if (refreshRate < 1.0)
         refreshRate = 60.0;
 
-    updatePrimaryOrientation();
-
-    filteredOrientation = orientation;
-    if (filteredOrientation == Qt::PrimaryOrientation)
-        filteredOrientation = primaryOrientation;
-
     updateHighDpi();
+    updatePrimaryOrientation(); // derived from the geometry
 }
 
 
@@ -149,6 +171,8 @@ QScreen::~QScreen()
 
 /*!
   Get the platform screen handle.
+
+  \sa {Qt Platform Abstraction}{Qt Platform Abstraction (QPA)}
 */
 QPlatformScreen *QScreen::handle() const
 {
@@ -233,6 +257,9 @@ QSize QScreen::size() const
   Depending on what information the underlying system provides the value might not be
   entirely accurate.
 
+  \note Physical DPI is expressed in device-independent dots. Multiply by QScreen::devicePixelRatio()
+  to get device-dependent density.
+
   \sa physicalDotsPerInchY()
 */
 qreal QScreen::physicalDotsPerInchX() const
@@ -248,6 +275,9 @@ qreal QScreen::physicalDotsPerInchX() const
   Depending on what information the underlying system provides the value might not be
   entirely accurate.
 
+  \note Physical DPI is expressed in device-independent dots. Multiply by QScreen::devicePixelRatio()
+  to get device-dependent density.
+
   \sa physicalDotsPerInchX()
 */
 qreal QScreen::physicalDotsPerInchY() const
@@ -266,6 +296,9 @@ qreal QScreen::physicalDotsPerInchY() const
   This is a convenience property that's simply the average of the physicalDotsPerInchX
   and physicalDotsPerInchY properties.
 
+  \note Physical DPI is expressed in device-independent dots. Multiply by QScreen::devicePixelRatio()
+  to get device-dependent density.
+
   \sa physicalDotsPerInchX()
   \sa physicalDotsPerInchY()
 */
@@ -490,58 +523,25 @@ QRect QScreen::availableVirtualGeometry() const
     return result;
 }
 
-/*!
-    Sets the orientations that the application is interested in receiving
-    updates for in conjunction with this screen.
-
-    For example, to receive orientation() updates and thus have
-    orientationChanged() signals being emitted for LandscapeOrientation and
-    InvertedLandscapeOrientation, call setOrientationUpdateMask() with
-    \a{mask} set to Qt::LandscapeOrientation | Qt::InvertedLandscapeOrientation.
-
-    The default, 0, means no orientationChanged() signals are fired.
-*/
-void QScreen::setOrientationUpdateMask(Qt::ScreenOrientations mask)
-{
-    Q_D(QScreen);
-    d->orientationUpdateMask = mask;
-    d->platformScreen->setOrientationUpdateMask(mask);
-    QGuiApplicationPrivate::updateFilteredScreenOrientation(this);
-}
-
-/*!
-    Returns the currently set orientation update mask.
-
-    \sa setOrientationUpdateMask()
-*/
-Qt::ScreenOrientations QScreen::orientationUpdateMask() const
-{
-    Q_D(const QScreen);
-    return d->orientationUpdateMask;
-}
-
 /*!
     \property QScreen::orientation
     \brief the screen orientation
 
-    The screen orientation represents the physical orientation
-    of the display. For example, the screen orientation of a mobile device
-    will change based on how it is being held. A change to the orientation
-    might or might not trigger a change to the primary orientation of the screen.
+    The \c orientation property tells the orientation of the screen from the
+    window system perspective.
 
-    Changes to this property will be filtered by orientationUpdateMask(),
-    so in order to receive orientation updates the application must first
-    call setOrientationUpdateMask() with a mask of the orientations it wants
-    to receive.
+    Most mobile devices and tablet computers contain accelerometer sensors.
+    The Qt Sensors module provides the ability to read this sensor directly.
+    However, the windowing system may rotate the entire screen automatically
+    based on how it is being held; in that case, this \c orientation property
+    will change.
 
-    Qt::PrimaryOrientation is never returned.
-
-    \sa primaryOrientation()
+    \sa primaryOrientation(), QWindow::contentOrientation()
 */
 Qt::ScreenOrientation QScreen::orientation() const
 {
     Q_D(const QScreen);
-    return d->filteredOrientation;
+    return d->orientation;
 }
 
 /*!
@@ -699,13 +699,33 @@ void QScreenPrivate::updatePrimaryOrientation()
     primaryOrientation = geometry.width() >= geometry.height() ? Qt::LandscapeOrientation : Qt::PortraitOrientation;
 }
 
+/*!
+    Returns the screen at \a point within the set of \l QScreen::virtualSiblings(),
+    or \c nullptr if outside of any screen.
+
+    The \a point is in relation to the virtualGeometry() of each set of virtual
+    siblings.
+
+    \since 5.15
+*/
+QScreen *QScreen::virtualSiblingAt(QPoint point)
+{
+    const auto &siblings = virtualSiblings();
+    for (QScreen *sibling : siblings) {
+        if (sibling->geometry().contains(point))
+            return sibling;
+    }
+    return nullptr;
+}
+
 /*!
     Creates and returns a pixmap constructed by grabbing the contents
     of the given \a window restricted by QRect(\a x, \a y, \a width,
-    \a height).
+    \a height). If \a window is 0, then the entire screen will be
+    grabbed.
 
     The arguments (\a{x}, \a{y}) specify the offset in the window,
-    whereas (\a{width}, \a{height}) specify the area to be copied.  If
+    whereas (\a{width}, \a{height}) specify the area to be copied. If
     \a width is negative, the function copies everything to the right
     border of the window. If \a height is negative, the function
     copies everything to the bottom of the window.
