@@ -36,6 +36,10 @@
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
+
+#include <AppKit/AppKit.h>
+#include <QuartzCore/QuartzCore.h>
+
 #include "qcocoawindow.h"
 #include "qcocoaintegration.h"
 #include "qcocoascreen.h"
@@ -57,9 +61,6 @@
 #include <QtGui/private/qcoregraphics_p.h>
 #include <QtGui/private/qhighdpiscaling_p.h>
 
-#include <AppKit/AppKit.h>
-#include <QuartzCore/QuartzCore.h>
-
 #include <QDebug>
 
 #include <vector>
@@ -71,14 +72,6 @@ enum {
     defaultWindowHeight = 160
 };
 
-static void qt_closePopups()
-{
-    while (QCocoaWindow *popup = QCocoaIntegration::instance()->popPopupWindow()) {
-        QWindowSystemInterface::handleCloseEvent(popup->window());
-        QWindowSystemInterface::flushWindowSystemEvents();
-    }
-}
-
 Q_LOGGING_CATEGORY(lcCocoaNotifications, "qt.qpa.cocoa.notifications");
 
 static void qRegisterNotificationCallbacks()
@@ -87,7 +80,7 @@ static void qRegisterNotificationCallbacks()
 
     NSNotificationCenter *center = [NSNotificationCenter defaultCenter];
 
-    const QMetaObject *metaObject = QMetaType::metaObjectForType(qRegisterMetaType<QCocoaWindow*>());
+    const QMetaObject *metaObject = QMetaType(qRegisterMetaType<QCocoaWindow*>()).metaObject();
     Q_ASSERT(metaObject);
 
     for (int i = 0; i < metaObject->methodCount(); ++i) {
@@ -152,12 +145,11 @@ QCocoaWindow::QCocoaWindow(QWindow *win, WId nativeHandle)
     , m_inSetGeometry(false)
     , m_inSetStyleMask(false)
     , m_menubar(nullptr)
-    , m_needsInvalidateShadow(false)
     , m_frameStrutEventsEnabled(false)
     , m_registerTouchCount(0)
     , m_resizableTransientParent(false)
     , m_alertRequest(NoAlertRequest)
-    , monitor(nil)
+    , m_monitor(nil)
     , m_drawContentBorderGradient(false)
     , m_topContentBorderThickness(0)
     , m_bottomContentBorderThickness(0)
@@ -179,10 +171,18 @@ void QCocoaWindow::initialize()
     if (!m_view)
         m_view = [[QNSView alloc] initWithCocoaWindow:this];
 
-    setGeometry(initialGeometry(window(), windowGeometry(), defaultWindowWidth, defaultWindowHeight));
+    // Compute the initial geometry based on the geometry set on the
+    // QWindow. This geometry has already been reflected to the
+    // QPlatformWindow in the constructor, so to ensure that the
+    // resulting setGeometry call does not think the geometry has
+    // already been applied, we reset the QPlatformWindow's view
+    // of the geometry first.
+    auto initialGeometry = QPlatformWindow::initialGeometry(window(),
+        windowGeometry(), defaultWindowWidth, defaultWindowHeight);
+    QPlatformWindow::d_ptr->rect = QRect();
+    setGeometry(initialGeometry);
 
     recreateWindowIfNeeded();
-    window()->setGeometry(geometry());
 
     m_initialized = true;
 }
@@ -223,14 +223,7 @@ QCocoaWindow::~QCocoaWindow()
 
 QSurfaceFormat QCocoaWindow::format() const
 {
-    QSurfaceFormat format = window()->requestedFormat();
-
-    // Upgrade the default surface format to include an alpha channel. The default RGB format
-    // causes Cocoa to spend an unreasonable amount of time converting it to RGBA internally.
-    if (format.alphaBufferSize() < 0)
-        format.setAlphaBufferSize(8);
-
-    return format;
+    return window()->requestedFormat();
 }
 
 void QCocoaWindow::setGeometry(const QRect &rectIn)
@@ -299,6 +292,25 @@ void QCocoaWindow::setCocoaGeometry(const QRect &rect)
     // will call QPlatformWindow::setGeometry(rect) during resize confirmation (see qnsview.mm)
 }
 
+bool QCocoaWindow::startSystemMove()
+{
+    switch (NSApp.currentEvent.type) {
+    case NSEventTypeLeftMouseDown:
+    case NSEventTypeRightMouseDown:
+    case NSEventTypeOtherMouseDown:
+    case NSEventTypeMouseMoved:
+    case NSEventTypeLeftMouseDragged:
+    case NSEventTypeRightMouseDragged:
+    case NSEventTypeOtherMouseDragged:
+        // The documentation only describes starting a system move
+        // based on mouse down events, but move events also work.
+        [m_view.window performWindowDragWithEvent:NSApp.currentEvent];
+        return true;
+    default:
+        return false;
+    }
+}
+
 void QCocoaWindow::setVisible(bool visible)
 {
     qCDebug(lcQpaWindow) << "QCocoaWindow::setVisible" << window() << visible;
@@ -360,8 +372,18 @@ void QCocoaWindow::setVisible(bool visible)
                 } else if (window()->modality() == Qt::ApplicationModal) {
                     // Show the window as application modal
                     eventDispatcher()->beginModalSession(window());
-                } else if (m_view.window.canBecomeKeyWindow && !eventDispatcher()->hasModalSession()) {
-                    [m_view.window makeKeyAndOrderFront:nil];
+                } else if (m_view.window.canBecomeKeyWindow) {
+                    bool shouldBecomeKeyNow = !NSApp.modalWindow || m_view.window.worksWhenModal;
+
+                    // Panels with becomesKeyOnlyIfNeeded set should not activate until a view
+                    // with needsPanelToBecomeKey, for example a line edit, is clicked.
+                    if ([m_view.window isKindOfClass:[NSPanel class]])
+                        shouldBecomeKeyNow &= !(static_cast<NSPanel*>(m_view.window).becomesKeyOnlyIfNeeded);
+
+                    if (shouldBecomeKeyNow)
+                        [m_view.window makeKeyAndOrderFront:nil];
+                    else
+                        [m_view.window orderFront:nil];
                 } else {
                     [m_view.window orderFront:nil];
                 }
@@ -371,7 +393,7 @@ void QCocoaWindow::setVisible(bool visible)
                     removeMonitor();
                     NSEventMask eventMask = NSEventMaskLeftMouseDown | NSEventMaskRightMouseDown
                                           | NSEventMaskOtherMouseDown | NSEventMaskMouseMoved;
-                    monitor = [NSEvent addGlobalMonitorForEventsMatchingMask:eventMask handler:^(NSEvent *e) {
+                    m_monitor = [NSEvent addGlobalMonitorForEventsMatchingMask:eventMask handler:^(NSEvent *e) {
                         const auto button = cocoaButton2QtButton(e);
                         const auto buttons = currentlyPressedMouseButtons();
                         const auto eventType = cocoaEvent2QtMouseEvent(e);
@@ -567,7 +589,10 @@ void QCocoaWindow::setWindowFlags(Qt::WindowFlags flags)
     Qt::WindowType type = static_cast<Qt::WindowType>(int(flags & Qt::WindowType_Mask));
     if ((type & Qt::Popup) != Qt::Popup && (type & Qt::Dialog) != Qt::Dialog) {
         NSWindowCollectionBehavior behavior = m_view.window.collectionBehavior;
-        if ((flags & Qt::WindowFullscreenButtonHint) || m_view.window.qt_fullScreen) {
+        const bool enableFullScreen = m_view.window.qt_fullScreen
+                                    || !(flags & Qt::CustomizeWindowHint)
+                                    || (flags & Qt::WindowFullscreenButtonHint);
+        if (enableFullScreen) {
             behavior |= NSWindowCollectionBehaviorFullScreenPrimary;
             behavior &= ~NSWindowCollectionBehaviorFullScreenAuxiliary;
         } else {
@@ -800,6 +825,11 @@ void QCocoaWindow::windowDidExitFullScreen()
     }
 }
 
+void QCocoaWindow::windowWillMiniaturize()
+{
+    QCocoaIntegration::instance()->closePopups(window());
+}
+
 void QCocoaWindow::windowDidMiniaturize()
 {
     if (!isContentView())
@@ -876,12 +906,11 @@ void QCocoaWindow::setWindowIcon(const QIcon &icon)
     QMacAutoReleasePool pool;
 
     if (icon.isNull()) {
-        NSWorkspace *workspace = [NSWorkspace sharedWorkspace];
-        [iconButton setImage:[workspace iconForFile:m_view.window.representedFilename]];
+        iconButton.image = [NSWorkspace.sharedWorkspace iconForFile:m_view.window.representedFilename];
     } else {
-        QPixmap pixmap = icon.pixmap(QSize(22, 22));
-        NSImage *image = static_cast<NSImage *>(qt_mac_create_nsimage(pixmap));
-        [iconButton setImage:[image autorelease]];
+        // Fall back to a size that looks good on the highest resolution screen available
+        auto fallbackSize = iconButton.frame.size.height * qGuiApp->devicePixelRatio();
+        iconButton.image = [NSImage imageFromQIcon:icon withSize:fallbackSize];
     }
 }
 
@@ -1022,27 +1051,15 @@ void QCocoaWindow::setMask(const QRegion &region)
 {
     qCDebug(lcQpaWindow) << "QCocoaWindow::setMask" << window() << region;
 
-    if (m_view.layer) {
-        if (!region.isEmpty()) {
-            QCFType<CGMutablePathRef> maskPath = CGPathCreateMutable();
-            for (const QRect &r : region)
-                CGPathAddRect(maskPath, nullptr, r.toCGRect());
-            CAShapeLayer *maskLayer = [CAShapeLayer layer];
-            maskLayer.path = maskPath;
-            m_view.layer.mask = maskLayer;
-        } else {
-            m_view.layer.mask = nil;
-        }
+    if (!region.isEmpty()) {
+        QCFType<CGMutablePathRef> maskPath = CGPathCreateMutable();
+        for (const QRect &r : region)
+            CGPathAddRect(maskPath, nullptr, r.toCGRect());
+        CAShapeLayer *maskLayer = [CAShapeLayer layer];
+        maskLayer.path = maskPath;
+        m_view.layer.mask = maskLayer;
     } else {
-        if (isContentView()) {
-            // Setting the mask requires invalidating the NSWindow shadow, but that needs
-            // to happen after the backingstore has been redrawn, so that AppKit can pick
-            // up the new window shape based on the backingstore content. Doing a display
-            // directly here is not an option, as the window might not be exposed at this
-            // time, and so would not result in an updated backingstore.
-            m_needsInvalidateShadow = true;
-            [m_view setNeedsDisplay:YES];
-        }
+        m_view.layer.mask = nil;
     }
 }
 
@@ -1079,11 +1096,9 @@ void QCocoaWindow::setParent(const QPlatformWindow *parentWindow)
 {
     qCDebug(lcQpaWindow) << "QCocoaWindow::setParent" << window() << (parentWindow ? parentWindow->window() : 0);
 
-    // recreate the window for compatibility
-    bool unhideAfterRecreate = parentWindow && !isEmbedded() && ![m_view isHidden];
+    // Recreate in case we need to get rid of a NSWindow, or create one
     recreateWindowIfNeeded();
-    if (unhideAfterRecreate)
-        [m_view setHidden:NO];
+
     setCocoaGeometry(geometry());
 }
 
@@ -1138,7 +1153,7 @@ void QCocoaWindow::viewDidChangeGlobalFrame()
 void QCocoaWindow::windowWillMove()
 {
     // Close any open popups on window move
-    qt_closePopups();
+    QCocoaIntegration::instance()->closePopups();
 }
 
 void QCocoaWindow::windowDidMove()
@@ -1186,8 +1201,17 @@ void QCocoaWindow::windowDidBecomeKey()
         QWindowSystemInterface::handleEnterEvent(m_enterLeaveTargetWindow, windowPoint, screenPoint);
     }
 
-    if (!windowIsPopupType())
-        QWindowSystemInterface::handleWindowActivated<QWindowSystemInterface::SynchronousDelivery>(window());
+    QNSView *firstResponderView = qt_objc_cast<QNSView *>(m_view.window.firstResponder);
+    if (!firstResponderView)
+        return;
+
+    const QCocoaWindow *focusCocoaWindow = firstResponderView.platformWindow;
+    if (focusCocoaWindow->windowIsPopupType())
+        return;
+
+    // See also [QNSView becomeFirstResponder]
+    QWindowSystemInterface::handleWindowActivated<QWindowSystemInterface::SynchronousDelivery>(
+                focusCocoaWindow->window(), Qt::ActiveWindowFocusReason);
 }
 
 void QCocoaWindow::windowDidResignKey()
@@ -1198,14 +1222,18 @@ void QCocoaWindow::windowDidResignKey()
     if (isForeignWindow())
         return;
 
-    // Key window will be non-nil if another window became key, so do not
-    // set the active window to zero here -- the new key window's
-    // NSWindowDidBecomeKeyNotification hander will change the active window.
-    NSWindow *keyWindow = [NSApp keyWindow];
-    if (!keyWindow || keyWindow == m_view.window) {
-        // No new key window, go ahead and set the active window to zero
-        if (!windowIsPopupType())
-            QWindowSystemInterface::handleWindowActivated<QWindowSystemInterface::SynchronousDelivery>(0);
+    // The current key window will be non-nil if another window became key. If that
+    // window is a Qt window, we delay the window activation event until the didBecomeKey
+    // notification is delivered to the active window, to ensure an atomic update.
+    NSWindow *newKeyWindow = [NSApp keyWindow];
+    if (newKeyWindow && newKeyWindow != m_view.window
+        && [newKeyWindow conformsToProtocol:@protocol(QNSWindowProtocol)])
+        return;
+
+    // Lost key window, go ahead and set the active window to zero
+    if (!windowIsPopupType()) {
+        QWindowSystemInterface::handleWindowActivated<QWindowSystemInterface::SynchronousDelivery>(
+            nullptr, Qt::ActiveWindowFocusReason);
     }
 }
 
@@ -1221,7 +1249,9 @@ void QCocoaWindow::windowDidOrderOffScreen()
 
 void QCocoaWindow::windowDidChangeOcclusionState()
 {
-    if (m_view.window.occlusionState & NSWindowOcclusionStateVisible)
+    bool visible = m_view.window.occlusionState & NSWindowOcclusionStateVisible;
+    qCDebug(lcQpaWindow) << "QCocoaWindow::windowDidChangeOcclusionState" << window() << "is now" << (visible ? "visible" : "occluded");
+    if (visible)
         [m_view setNeedsDisplay:YES];
     else
         handleExposeEvent(QRegion());
@@ -1262,23 +1292,12 @@ void QCocoaWindow::windowDidChangeScreen()
         currentScreen->requestUpdate();
     }
 }
-/*
-    The window's backing scale factor or color space has changed.
-*/
-void QCocoaWindow::windowDidChangeBackingProperties()
-{
-    // Ideally we would plumb this thought QPA in a way that lets clients
-    // invalidate their own caches, and recreate QBackingStore. For now we
-    // trigger an expose, and let QCocoaBackingStore deal with its own
-    // buffer invalidation.
-    [m_view setNeedsDisplay:YES];
-}
 
 void QCocoaWindow::windowWillClose()
 {
     // Close any open popups on window closing.
     if (window() && !windowIsPopupType(window()->type()))
-        qt_closePopups();
+        QCocoaIntegration::instance()->closePopups();
 }
 
 // ----------------------- NSWindowDelegate callbacks -----------------------
@@ -1286,14 +1305,11 @@ void QCocoaWindow::windowWillClose()
 bool QCocoaWindow::windowShouldClose()
 {
     qCDebug(lcQpaWindow) << "QCocoaWindow::windowShouldClose" << window();
-   // This callback should technically only determine if the window
-   // should (be allowed to) close, but since our QPA API to determine
-   // that also involves actually closing the window we do both at the
-   // same time, instead of doing the latter in windowWillClose.
-    bool accepted = false;
-    QWindowSystemInterface::handleCloseEvent(window(), &accepted);
-    QWindowSystemInterface::flushWindowSystemEvents();
-    return accepted;
+    // This callback should technically only determine if the window
+    // should (be allowed to) close, but since our QPA API to determine
+    // that also involves actually closing the window we do both at the
+    // same time, instead of doing the latter in windowWillClose.
+    return QWindowSystemInterface::handleCloseEvent<QWindowSystemInterface::SynchronousDelivery>(window());
 }
 
 // ----------------------------- QPA forwarding -----------------------------
@@ -1441,11 +1457,6 @@ void QCocoaWindow::recreateWindowIfNeeded()
     if ((isContentView() && !shouldBeContentView) || (recreateReason & PanelChanged)) {
         if (m_nsWindow) {
             qCDebug(lcQpaWindow) << "Getting rid of existing window" << m_nsWindow;
-            if (m_nsWindow.observationInfo) {
-                qCCritical(lcQpaWindow) << m_nsWindow << "has active key-value observers (KVO)!"
-                    << "These will stop working now that the window is recreated, and will result in exceptions"
-                    << "when the observers are removed. Break in QCocoaWindow::recreateWindowIfNeeded to debug.";
-            }
             [m_nsWindow closeAndRelease];
             if (isContentView() && !isEmbeddedView) {
                 // We explicitly disassociate m_view from the window's contentView,
@@ -1481,15 +1492,8 @@ void QCocoaWindow::recreateWindowIfNeeded()
         setWindowFilePath(window()->filePath()); // Also sets window icon
         setWindowState(window()->windowState());
     } else {
-        // Child windows have no NSWindow, link the NSViews instead.
+        // Child windows have no NSWindow, re-parent to superview instead
         [parentCocoaWindow->m_view addSubview:m_view];
-        QRect rect = windowGeometry();
-        // Prevent setting a (0,0) window size; causes opengl context
-        // "Invalid Drawable" warnings.
-        if (rect.isNull())
-            rect.setSize(QSize(1, 1));
-        NSRect frame = NSMakeRect(rect.x(), rect.y(), rect.width(), rect.height());
-        [m_view setFrame:frame];
         [m_view setHidden:!window()->isVisible()];
     }
 
@@ -1526,17 +1530,6 @@ bool QCocoaWindow::updatesWithDisplayLink() const
 
 void QCocoaWindow::deliverUpdateRequest()
 {
-    // Don't send update requests for views that need display, as the update
-    // request doesn't carry any information about dirty rects, so the app
-    // may end up painting a smaller region than required. (For some reason
-    // the layer and view's needsDisplay status isn't always in sync, even if
-    // the view is layer-backed, not layer-hosted, so we check both).
-    if (m_view.layer.needsDisplay || m_view.needsDisplay) {
-        qCDebug(lcQpaDrawing) << "View needs display, deferring update request for" << window();
-        requestUpdate();
-        return;
-    }
-
     qCDebug(lcQpaDrawing) << "Delivering update request to" << window();
     QPlatformWindow::deliverUpdateRequest();
 }
@@ -1641,6 +1634,7 @@ QCocoaNSWindow *QCocoaWindow::createNSWindow(bool shouldBePanel)
 
     nsWindow.restorable = NO;
     nsWindow.level = windowLevel(flags);
+    nsWindow.tabbingMode = NSWindowTabbingModeDisallowed;
 
     if (shouldBePanel) {
         // Qt::Tool windows hide on app deactivation, unless Qt::WA_MacAlwaysShowToolWindow is set
@@ -1660,7 +1654,7 @@ QCocoaNSWindow *QCocoaWindow::createNSWindow(bool shouldBePanel)
 
     applyContentBorderThickness(nsWindow);
 
-    if (format().colorSpace() == QSurfaceFormat::sRGBColorSpace)
+    if (format().colorSpace() == QColorSpace::SRgb)
         nsWindow.colorSpace = NSColorSpace.sRGBColorSpace;
 
     return nsWindow;
@@ -1673,10 +1667,10 @@ bool QCocoaWindow::alwaysShowToolWindow() const
 
 void QCocoaWindow::removeMonitor()
 {
-    if (!monitor)
+    if (!m_monitor)
         return;
-    [NSEvent removeMonitor:monitor];
-    monitor = nil;
+    [NSEvent removeMonitor:m_monitor];
+    m_monitor = nil;
 }
 
 bool QCocoaWindow::setWindowModified(bool modified)
@@ -1717,9 +1711,9 @@ void QCocoaWindow::registerTouch(bool enable)
 {
     m_registerTouchCount += enable ? 1 : -1;
     if (enable && m_registerTouchCount == 1)
-        [m_view setAcceptsTouchEvents:YES];
+        m_view.allowedTouchTypes |= NSTouchTypeMaskIndirect;
     else if (m_registerTouchCount == 0)
-        [m_view setAcceptsTouchEvents:NO];
+        m_view.allowedTouchTypes &= ~NSTouchTypeMaskIndirect;
 }
 
 void QCocoaWindow::setContentBorderThickness(int topThickness, int bottomThickness)
@@ -1822,8 +1816,17 @@ void QCocoaWindow::updateNSToolbar()
 
 bool QCocoaWindow::testContentBorderAreaPosition(int position) const
 {
-    return isContentView() && m_drawContentBorderGradient &&
-            0 <= position && position < [m_view.window contentBorderThicknessForEdge:NSMaxYEdge];
+    if (!m_drawContentBorderGradient || !isContentView())
+        return false;
+
+    // Determine if the given y postion (relative to the content area) is inside the
+    // unified toolbar area. Note that the value returned by contentBorderThicknessForEdge
+    // includes the title bar height; subtract it.
+    const int contentBorderThickness = [m_view.window contentBorderThicknessForEdge:NSMaxYEdge];
+    const NSRect frameRect = m_view.window.frame;
+    const NSRect contentRect = [m_view.window contentRectForFrameRect:frameRect];
+    const CGFloat titlebarHeight = frameRect.size.height - contentRect.size.height;
+    return 0 <= position && position < (contentBorderThickness - titlebarHeight);
 }
 
 qreal QCocoaWindow::devicePixelRatio() const
@@ -1840,7 +1843,7 @@ qreal QCocoaWindow::devicePixelRatio() const
 QWindow *QCocoaWindow::childWindowAt(QPoint windowPoint)
 {
     QWindow *targetWindow = window();
-    foreach (QObject *child, targetWindow->children())
+    for (QObject *child : targetWindow->children())
         if (QWindow *childWindow = qobject_cast<QWindow *>(child))
             if (QPlatformWindow *handle = childWindow->handle())
                 if (handle->isExposed() && childWindow->geometry().contains(windowPoint))
@@ -1866,13 +1869,6 @@ bool QCocoaWindow::shouldRefuseKeyWindowAndFirstResponder()
     return false;
 }
 
-QPoint QCocoaWindow::bottomLeftClippedByNSWindowOffsetStatic(QWindow *window)
-{
-    if (window->handle())
-        return static_cast<QCocoaWindow *>(window->handle())->bottomLeftClippedByNSWindowOffset();
-    return QPoint();
-}
-
 QPoint QCocoaWindow::bottomLeftClippedByNSWindowOffset() const
 {
     if (!m_view)
