@@ -145,12 +145,11 @@
 
 #include "QtWidgets/qscrollbar.h"
 #include "QtCore/qdir.h"
+#if QT_CONFIG(stringlistmodel)
 #include "QtCore/qstringlistmodel.h"
-#if QT_CONFIG(dirmodel)
-#include "QtWidgets/qdirmodel.h"
 #endif
 #if QT_CONFIG(filesystemmodel)
-#include "QtWidgets/qfilesystemmodel.h"
+#include "QtGui/qfilesystemmodel.h"
 #endif
 #include "QtWidgets/qheaderview.h"
 #if QT_CONFIG(listview)
@@ -158,8 +157,8 @@
 #endif
 #include "QtWidgets/qapplication.h"
 #include "QtGui/qevent.h"
-#include "QtWidgets/qdesktopwidget.h"
-#include <private/qdesktopwidget_p.h>
+#include <private/qapplication_p.h>
+#include <private/qwidget_p.h>
 #if QT_CONFIG(lineedit)
 #include "QtWidgets/qlineedit.h"
 #endif
@@ -470,19 +469,13 @@ QMatchData QCompletionEngine::filterHistory()
     if (curParts.count() <= 1 || c->proxy->showAll || !source)
         return QMatchData();
 
-#if QT_CONFIG(dirmodel)
-    const bool isDirModel = (qobject_cast<QDirModel *>(source) != nullptr);
-#else
-    const bool isDirModel = false;
-#endif
-    Q_UNUSED(isDirModel)
 #if QT_CONFIG(filesystemmodel)
     const bool isFsModel = (qobject_cast<QFileSystemModel *>(source) != nullptr);
 #else
     const bool isFsModel = false;
 #endif
-    Q_UNUSED(isFsModel)
-    QVector<int> v;
+    Q_UNUSED(isFsModel);
+    QList<int> v;
     QIndexMapper im(v);
     QMatchData m(im, -1, true);
 
@@ -490,7 +483,7 @@ QMatchData QCompletionEngine::filterHistory()
         QString str = source->index(i, c->column).data().toString();
         if (str.startsWith(c->prefix, c->cs)
 #if !defined(Q_OS_WIN)
-            && ((!isFsModel && !isDirModel) || QDir::toNativeSeparators(str) != QDir::separator())
+            && (!isFsModel || QDir::toNativeSeparators(str) != QDir::separator())
 #endif
             )
             m.indices.append(i);
@@ -594,7 +587,8 @@ QIndexMapper QSortedModelEngine::indexHint(QString part, const QModelIndex& pare
     const CacheItem::const_iterator it = map.lowerBound(part);
 
     // look backward for first valid hint
-    for(CacheItem::const_iterator it1 = it; it1-- != map.constBegin();) {
+    for (CacheItem::const_iterator it1 = it; it1 != map.constBegin();) {
+        --it1;
         const QMatchData& value = it1.value();
         if (value.isValid()) {
             if (order == Qt::AscendingOrder) {
@@ -754,7 +748,7 @@ int QUnsortedModelEngine::buildIndices(const QString& str, const QModelIndex& pa
         case Qt::MatchExactly:
         case Qt::MatchFixedString:
         case Qt::MatchCaseSensitive:
-        case Qt::MatchRegExp:
+        case Qt::MatchRegularExpression:
         case Qt::MatchWildcard:
         case Qt::MatchWrap:
         case Qt::MatchRecursive:
@@ -790,7 +784,7 @@ QMatchData QUnsortedModelEngine::filter(const QString& part, const QModelIndex&
 {
     QMatchData hint;
 
-    QVector<int> v;
+    QList<int> v;
     QIndexMapper im(v);
     QMatchData m(im, -1, true);
 
@@ -900,13 +894,6 @@ void QCompleterPrivate::_q_complete(QModelIndex index, bool highlighted)
         QModelIndex si = proxy->mapToSource(index);
         si = si.sibling(si.row(), column); // for clicked()
         completion = q->pathFromIndex(si);
-#if QT_CONFIG(dirmodel)
-        // add a trailing separator in inline
-        if (mode == QCompleter::InlineCompletion) {
-            if (qobject_cast<QDirModel *>(proxy->sourceModel()) && QFileInfo(completion).isDir())
-                completion += QDir::separator();
-        }
-#endif
 #if QT_CONFIG(filesystemmodel)
         // add a trailing separator in inline
         if (mode == QCompleter::InlineCompletion) {
@@ -934,7 +921,7 @@ void QCompleterPrivate::_q_autoResizePopup()
 
 void QCompleterPrivate::showPopup(const QRect& rect)
 {
-    const QRect screen = QDesktopWidgetPrivate::availableGeometry(widget);
+    const QRect screen = QWidgetPrivate::availableScreenGeometry(widget);
     Qt::LayoutDirection dir = widget->layoutDirection();
     QPoint pos;
     int rh, w;
@@ -976,18 +963,48 @@ void QCompleterPrivate::showPopup(const QRect& rect)
         popup->show();
 }
 
+#if QT_CONFIG(filesystemmodel)
+static bool isRoot(const QFileSystemModel *model, const QString &path)
+{
+    const auto index = model->index(path);
+    return index.isValid() && model->fileInfo(index).isRoot();
+}
+
+static bool completeOnLoaded(const QFileSystemModel *model,
+                             const QString &nativePrefix,
+                             const QString &path,
+                             Qt::CaseSensitivity caseSensitivity)
+{
+    const auto pathSize = path.size();
+    const auto prefixSize = nativePrefix.size();
+    if (prefixSize < pathSize)
+        return false;
+    const QString prefix = QDir::fromNativeSeparators(nativePrefix);
+    if (prefixSize == pathSize)
+        return path.compare(prefix, caseSensitivity) == 0 && isRoot(model, path);
+    // The user is typing something within that directory and is not in a subdirectory yet.
+    const auto separator = QLatin1Char('/');
+    return prefix.startsWith(path, caseSensitivity) && prefix.at(pathSize) == separator
+        && !QStringView{prefix}.right(prefixSize - pathSize - 1).contains(separator);
+}
+
 void QCompleterPrivate::_q_fileSystemModelDirectoryLoaded(const QString &path)
 {
     Q_Q(QCompleter);
     // Slot called when QFileSystemModel has finished loading.
     // If we hide the popup because there was no match because the model was not loaded yet,
-    // we re-start the completion when we get the results
-    if (hiddenBecauseNoMatch
-        && prefix.startsWith(path) && prefix != (path + QLatin1Char('/'))
-        && widget) {
-        q->complete();
+    // we re-start the completion when we get the results (unless triggered by
+    // something else, see QTBUG-14292).
+    if (hiddenBecauseNoMatch && widget) {
+        if (auto model = qobject_cast<const QFileSystemModel *>(proxy->sourceModel())) {
+            if (completeOnLoaded(model, prefix, path, cs))
+                q->complete();
+        }
     }
 }
+#else // QT_CONFIG(filesystemmodel)
+void QCompleterPrivate::_q_fileSystemModelDirectoryLoaded(const QString &) {}
+#endif
 
 /*!
     Constructs a completer object with the given \a parent.
@@ -1092,15 +1109,6 @@ void QCompleter::setModel(QAbstractItemModel *model)
         setPopup(d->popup); // set the model and make new connections
     if (oldModel && oldModel->QObject::parent() == this)
         delete oldModel;
-#if QT_CONFIG(dirmodel)
-    if (qobject_cast<QDirModel *>(model)) {
-#if defined(Q_OS_WIN)
-        setCaseSensitivity(Qt::CaseInsensitive);
-#else
-        setCaseSensitivity(Qt::CaseSensitive);
-#endif
-    }
-#endif // QT_CONFIG(dirmodel)
 #if QT_CONFIG(filesystemmodel)
     QFileSystemModel *fsModel = qobject_cast<QFileSystemModel *>(model);
     if (fsModel) {
@@ -1229,7 +1237,7 @@ Qt::MatchFlags QCompleter::filterMode() const
 void QCompleter::setPopup(QAbstractItemView *popup)
 {
     Q_D(QCompleter);
-    Q_ASSERT(popup != 0);
+    Q_ASSERT(popup != nullptr);
     if (d->popup) {
         QObject::disconnect(d->popup->selectionModel(), nullptr, this, nullptr);
         QObject::disconnect(d->popup, nullptr, this, nullptr);
@@ -1384,7 +1392,7 @@ bool QCompleter::eventFilter(QObject *o, QEvent *e)
             // widget lost focus, hide the popup
             if (d->widget && (!d->widget->hasFocus()
 #ifdef QT_KEYPAD_NAVIGATION
-                || (QApplication::keypadNavigationEnabled() && !d->widget->hasEditFocus())
+                || (QApplicationPrivate::keypadNavigationEnabled() && !d->widget->hasEditFocus())
 #endif
                 ))
                 d->popup->hide();
@@ -1402,7 +1410,7 @@ bool QCompleter::eventFilter(QObject *o, QEvent *e)
         switch (key) {
 #ifdef QT_KEYPAD_NAVIGATION
         case Qt::Key_Select:
-            if (!QApplication::keypadNavigationEnabled())
+            if (!QApplicationPrivate::keypadNavigationEnabled())
                 break;
 #endif
         case Qt::Key_Return:
@@ -1432,7 +1440,7 @@ bool QCompleter::eventFilter(QObject *o, QEvent *e)
 #ifdef QT_KEYPAD_NAVIGATION
     case QEvent::KeyRelease: {
         QKeyEvent *ke = static_cast<QKeyEvent *>(e);
-        if (QApplication::keypadNavigationEnabled() && ke->key() == Qt::Key_Back) {
+        if (QApplicationPrivate::keypadNavigationEnabled() && ke->key() == Qt::Key_Back) {
             // Send the event to the 'widget'. This is what we did for KeyPress, so we need
             // to do the same for KeyRelease, in case the widget's KeyPress event set
             // up something (such as a timer) that is relying on also receiving the
@@ -1449,7 +1457,7 @@ bool QCompleter::eventFilter(QObject *o, QEvent *e)
 
     case QEvent::MouseButtonPress: {
 #ifdef QT_KEYPAD_NAVIGATION
-        if (QApplication::keypadNavigationEnabled()) {
+        if (QApplicationPrivate::keypadNavigationEnabled()) {
             // if we've clicked in the widget (or its descendant), let it handle the click
             QWidget *source = qobject_cast<QWidget *>(o);
             if (source) {
@@ -1474,7 +1482,7 @@ bool QCompleter::eventFilter(QObject *o, QEvent *e)
 
     case QEvent::InputMethod:
     case QEvent::ShortcutOverride:
-        QApplication::sendEvent(d->widget, e);
+        QCoreApplication::sendEvent(d->widget, e);
         break;
 
     default:
@@ -1811,26 +1819,19 @@ QString QCompleter::pathFromIndex(const QModelIndex& index) const
     QAbstractItemModel *sourceModel = d->proxy->sourceModel();
     if (!sourceModel)
         return QString();
-    bool isDirModel = false;
     bool isFsModel = false;
-#if QT_CONFIG(dirmodel)
-    isDirModel = qobject_cast<QDirModel *>(d->proxy->sourceModel()) != nullptr;
-#endif
 #if QT_CONFIG(filesystemmodel)
     isFsModel = qobject_cast<QFileSystemModel *>(d->proxy->sourceModel()) != nullptr;
 #endif
-    if (!isDirModel && !isFsModel)
+    if (!isFsModel)
         return sourceModel->data(index, d->role).toString();
 
     QModelIndex idx = index;
     QStringList list;
     do {
         QString t;
-        if (isDirModel)
-            t = sourceModel->data(idx, Qt::EditRole).toString();
 #if QT_CONFIG(filesystemmodel)
-        else
-            t = sourceModel->data(idx, QFileSystemModel::FileNameRole).toString();
+        t = sourceModel->data(idx, QFileSystemModel::FileNameRole).toString();
 #endif
         list.prepend(t);
         QModelIndex parent = idx.parent();
@@ -1860,20 +1861,13 @@ QString QCompleter::pathFromIndex(const QModelIndex& index) const
 */
 QStringList QCompleter::splitPath(const QString& path) const
 {
-    bool isDirModel = false;
     bool isFsModel = false;
-#if QT_CONFIG(dirmodel)
-    Q_D(const QCompleter);
-    isDirModel = qobject_cast<QDirModel *>(d->proxy->sourceModel()) != nullptr;
-#endif
 #if QT_CONFIG(filesystemmodel)
-#if !QT_CONFIG(dirmodel)
     Q_D(const QCompleter);
-#endif
     isFsModel = qobject_cast<QFileSystemModel *>(d->proxy->sourceModel()) != nullptr;
 #endif
 
-    if ((!isDirModel && !isFsModel) || path.isEmpty())
+    if (!isFsModel || path.isEmpty())
         return QStringList(completionPrefix());
 
     QString pathCopy = QDir::toNativeSeparators(path);
