@@ -1,7 +1,8 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
-** Copyright (C) 2016 Intel Corporation.
+** Copyright (C) 2019 The Qt Company Ltd.
+** Copyright (C) 2020 Intel Corporation.
+** Copyright (C) 2019 Klarälvdalens Datakonsult AB.
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
@@ -31,15 +32,16 @@
 #include "qsharedpointer.h"
 #include <QtTest/QtTest>
 #include <QtCore/QHash>
+#include <QtCore/QList>
 #include <QtCore/QMap>
 #include <QtCore/QThread>
-#include <QtCore/QVector>
 
 #include "externaltests.h"
 #include "forwarddeclared.h"
 #include "nontracked.h"
 #include "wrapper.h"
 
+#include <array>
 #include <memory>
 #include <stdlib.h>
 #include <time.h>
@@ -75,10 +77,9 @@ private slots:
     void functionCallDownCast();
     void upCast();
     void qobjectWeakManagement();
-    void noSharedPointerFromWeakQObject();
-    void sharedPointerFromQObjectWithWeak();
     void weakQObjectFromSharedPointer();
     void objectCast();
+    void objectCastStdSharedPtr();
     void differentPointers();
     void virtualBaseDifferentPointers();
 #ifndef QTEST_NO_RTTI
@@ -90,9 +91,8 @@ private slots:
 #endif
     void constCorrectness();
     void customDeleter();
-#ifdef Q_COMPILER_LAMBDA
     void lambdaCustomDeleter();
-#endif
+    void customDeleterOnNullptr();
     void creating();
     void creatingCvQualified();
     void creatingVariadic();
@@ -105,12 +105,15 @@ private slots:
     void sharedFromThis();
 
     void constructorThrow();
+    void overloads();
 
     void threadStressTest_data();
     void threadStressTest();
     void validConstructs();
     void invalidConstructs_data();
     void invalidConstructs();
+
+
     // let invalidConstructs be the last test, because it's the slowest;
     // add new tests above this block
 public slots:
@@ -148,7 +151,7 @@ QtSharedPointer::ExternalRefCountData *refCountData(const QSharedPointer<T> &b)
         QtSharedPointer::ExternalRefCountData* data;
     };
     // sanity checks:
-    Q_STATIC_ASSERT(sizeof(QSharedPointer<T>) == sizeof(Dummy));
+    static_assert(sizeof(QSharedPointer<T>) == sizeof(Dummy));
     Q_ASSERT(static_cast<const Dummy*>(static_cast<const void*>(&b))->value == b.data());
     return static_cast<const Dummy*>(static_cast<const void*>(&b))->data;
 }
@@ -227,7 +230,6 @@ struct NoDefaultConstructorConstRef2
     NoDefaultConstructorConstRef2(const QByteArray &ba, int i = 42) : str(QString::fromLatin1(ba)), i(i) {}
 };
 
-#ifdef Q_COMPILER_RVALUE_REFS
 struct NoDefaultConstructorRRef1
 {
     int &i;
@@ -239,7 +241,6 @@ struct NoDefaultConstructorRRef2
     std::unique_ptr<int> i;
     NoDefaultConstructorRRef2(std::unique_ptr<int> &&i) : i(std::move(i)) {}
 };
-#endif
 
 void tst_QSharedPointer::basics_data()
 {
@@ -297,8 +298,8 @@ void tst_QSharedPointer::basics()
         QVERIFY(! (ptr == otherData));
         QVERIFY(! (otherData == ptr));
     }
-    QVERIFY(!refCountData(ptr) || refCountData(ptr)->weakref.load() == 1);
-    QVERIFY(!refCountData(ptr) || refCountData(ptr)->strongref.load() == 1);
+    QVERIFY(!refCountData(ptr) || refCountData(ptr)->weakref.loadRelaxed() == 1);
+    QVERIFY(!refCountData(ptr) || refCountData(ptr)->strongref.loadRelaxed() == 1);
 
     {
         // create another object:
@@ -310,8 +311,8 @@ void tst_QSharedPointer::basics()
 
         // otherData is deleted here
     }
-    QVERIFY(!refCountData(ptr) || refCountData(ptr)->weakref.load() == 1);
-    QVERIFY(!refCountData(ptr) || refCountData(ptr)->strongref.load() == 1);
+    QVERIFY(!refCountData(ptr) || refCountData(ptr)->weakref.loadRelaxed() == 1);
+    QVERIFY(!refCountData(ptr) || refCountData(ptr)->strongref.loadRelaxed() == 1);
 
     {
         // create a copy:
@@ -328,8 +329,8 @@ void tst_QSharedPointer::basics()
         QCOMPARE(copy.get(), aData);
         QVERIFY(copy == aData);
     }
-    QVERIFY(!refCountData(ptr) || refCountData(ptr)->weakref.load() == 1);
-    QVERIFY(!refCountData(ptr) || refCountData(ptr)->strongref.load() == 1);
+    QVERIFY(!refCountData(ptr) || refCountData(ptr)->weakref.loadRelaxed() == 1);
+    QVERIFY(!refCountData(ptr) || refCountData(ptr)->strongref.loadRelaxed() == 1);
 
     {
         // create a weak reference:
@@ -361,8 +362,8 @@ void tst_QSharedPointer::basics()
         QCOMPARE(strong.data(), aData);
         QCOMPARE(strong.get(), aData);
     }
-    QVERIFY(!refCountData(ptr) || refCountData(ptr)->weakref.load() == 1);
-    QVERIFY(!refCountData(ptr) || refCountData(ptr)->strongref.load() == 1);
+    QVERIFY(!refCountData(ptr) || refCountData(ptr)->weakref.loadRelaxed() == 1);
+    QVERIFY(!refCountData(ptr) || refCountData(ptr)->strongref.loadRelaxed() == 1);
 
     // aData is deleted here
 }
@@ -508,7 +509,6 @@ void tst_QSharedPointer::swap()
 
 void tst_QSharedPointer::moveSemantics()
 {
-#ifdef Q_COMPILER_RVALUE_REFS
     QSharedPointer<int> p1, p2(new int(42)), control = p2;
     QVERIFY(p1 != control);
     QVERIFY(p1.isNull());
@@ -561,9 +561,6 @@ void tst_QSharedPointer::moveSemantics()
     QVERIFY(w1.isNull());
     QVERIFY(w2.isNull());
     QVERIFY(w3.isNull());
-#else
-    QSKIP("This test requires C++11 rvalue/move semantics support in the compiler.");
-#endif
 }
 
 void tst_QSharedPointer::useOfForwardDeclared()
@@ -580,10 +577,10 @@ void tst_QSharedPointer::useOfForwardDeclared()
 
     // move assignment:
     QSharedPointer<ForwardDeclared> sp4;
-    sp4 = qMove(sp);
+    sp4 = std::move(sp);
 
     // and move constuction:
-    QSharedPointer<ForwardDeclared> sp5 = qMove(sp2);
+    QSharedPointer<ForwardDeclared> sp5 = std::move(sp2);
 
     // swapping:
     sp4.swap(sp3);
@@ -850,15 +847,15 @@ void tst_QSharedPointer::upCast()
         QVERIFY(baseptr == derivedptr);
         QCOMPARE(static_cast<Data *>(derivedptr.data()), baseptr.data());
     }
-    QCOMPARE(int(refCountData(baseptr)->weakref.load()), 1);
-    QCOMPARE(int(refCountData(baseptr)->strongref.load()), 1);
+    QCOMPARE(int(refCountData(baseptr)->weakref.loadRelaxed()), 1);
+    QCOMPARE(int(refCountData(baseptr)->strongref.loadRelaxed()), 1);
 
     {
         QWeakPointer<DerivedData> derivedptr = qWeakPointerCast<DerivedData>(baseptr);
         QVERIFY(baseptr == derivedptr);
     }
-    QCOMPARE(int(refCountData(baseptr)->weakref.load()), 1);
-    QCOMPARE(int(refCountData(baseptr)->strongref.load()), 1);
+    QCOMPARE(int(refCountData(baseptr)->weakref.loadRelaxed()), 1);
+    QCOMPARE(int(refCountData(baseptr)->strongref.loadRelaxed()), 1);
 
     {
         QWeakPointer<Data> weakptr = baseptr;
@@ -866,16 +863,16 @@ void tst_QSharedPointer::upCast()
         QVERIFY(baseptr == derivedptr);
         QCOMPARE(static_cast<Data *>(derivedptr.data()), baseptr.data());
     }
-    QCOMPARE(int(refCountData(baseptr)->weakref.load()), 1);
-    QCOMPARE(int(refCountData(baseptr)->strongref.load()), 1);
+    QCOMPARE(int(refCountData(baseptr)->weakref.loadRelaxed()), 1);
+    QCOMPARE(int(refCountData(baseptr)->strongref.loadRelaxed()), 1);
 
     {
         QSharedPointer<DerivedData> derivedptr = baseptr.staticCast<DerivedData>();
         QVERIFY(baseptr == derivedptr);
         QCOMPARE(static_cast<Data *>(derivedptr.data()), baseptr.data());
     }
-    QCOMPARE(int(refCountData(baseptr)->weakref.load()), 1);
-    QCOMPARE(int(refCountData(baseptr)->strongref.load()), 1);
+    QCOMPARE(int(refCountData(baseptr)->weakref.loadRelaxed()), 1);
+    QCOMPARE(int(refCountData(baseptr)->strongref.loadRelaxed()), 1);
 }
 
 class OtherObject: public QObject
@@ -889,17 +886,18 @@ void tst_QSharedPointer::qobjectWeakManagement()
         QWeakPointer<QObject> weak;
         weak = QWeakPointer<QObject>();
         QVERIFY(weak.isNull());
-        QVERIFY(!weak.data());
+        QVERIFY(weak.toStrongRef().isNull());
     }
 
     {
         QObject *obj = new QObject;
-        QWeakPointer<QObject> weak(obj);
+        QSharedPointer<QObject> shared(obj);
+        QWeakPointer<QObject> weak(shared);
         QVERIFY(!weak.isNull());
-        QVERIFY(weak.data() == obj);
+        QVERIFY(weak.toStrongRef() == obj);
 
         // now delete
-        delete obj;
+        shared.reset();
         QVERIFY(weak.isNull());
     }
     safetyCheck();
@@ -907,41 +905,14 @@ void tst_QSharedPointer::qobjectWeakManagement()
     {
         // same, bit with operator=
         QObject *obj = new QObject;
+        QSharedPointer<QObject> shared(obj);
         QWeakPointer<QObject> weak;
-        weak = obj;
+        weak = shared;
         QVERIFY(!weak.isNull());
-        QVERIFY(weak.data() == obj);
+        QVERIFY(weak.toStrongRef() == obj);
 
         // now delete
-        delete obj;
-        QVERIFY(weak.isNull());
-    }
-    safetyCheck();
-
-    {
-        // delete triggered by parent
-        QObject *obj, *parent;
-        parent = new QObject;
-        obj = new QObject(parent);
-        QWeakPointer<QObject> weak(obj);
-
-        // now delete the parent
-        delete parent;
-        QVERIFY(weak.isNull());
-    }
-    safetyCheck();
-
-    {
-        // same as above, but set the parent after QWeakPointer is created
-        QObject *obj, *parent;
-        obj = new QObject;
-        QWeakPointer<QObject> weak(obj);
-
-        parent = new QObject;
-        obj->setParent(parent);
-
-        // now delete the parent
-        delete parent;
+        shared.reset();
         QVERIFY(weak.isNull());
     }
     safetyCheck();
@@ -949,16 +920,17 @@ void tst_QSharedPointer::qobjectWeakManagement()
     {
         // with two QWeakPointers
         QObject *obj = new QObject;
-        QWeakPointer<QObject> weak(obj);
+        QSharedPointer<QObject> shared(obj);
+        QWeakPointer<QObject> weak(shared);
 
         {
-            QWeakPointer<QObject> weak2(obj);
+            QWeakPointer<QObject> weak2(shared);
             QVERIFY(!weak2.isNull());
             QVERIFY(weak == weak2);
         }
         QVERIFY(!weak.isNull());
 
-        delete obj;
+        shared.reset();
         QVERIFY(weak.isNull());
     }
     safetyCheck();
@@ -966,13 +938,14 @@ void tst_QSharedPointer::qobjectWeakManagement()
     {
         // same, but delete the pointer while two QWeakPointers exist
         QObject *obj = new QObject;
-        QWeakPointer<QObject> weak(obj);
+        QSharedPointer<QObject> shared(obj);
+        QWeakPointer<QObject> weak(shared);
 
         {
-            QWeakPointer<QObject> weak2(obj);
+            QWeakPointer<QObject> weak2(shared);
             QVERIFY(!weak2.isNull());
 
-            delete obj;
+            shared.reset();
             QVERIFY(weak.isNull());
             QVERIFY(weak2.isNull());
         }
@@ -981,51 +954,17 @@ void tst_QSharedPointer::qobjectWeakManagement()
     safetyCheck();
 }
 
-void tst_QSharedPointer::noSharedPointerFromWeakQObject()
-{
-    // you're not allowed to create a QSharedPointer from an unmanaged QObject
-    QObject obj;
-    QWeakPointer<QObject> weak(&obj);
-
-    {
-        QTest::ignoreMessage(QtWarningMsg ,  "QSharedPointer: cannot create a QSharedPointer from a QObject-tracking QWeakPointer");
-        QSharedPointer<QObject> strong = weak.toStrongRef();
-        QVERIFY(strong.isNull());
-    }
-    {
-        QTest::ignoreMessage(QtWarningMsg ,  "QSharedPointer: cannot create a QSharedPointer from a QObject-tracking QWeakPointer");
-        QSharedPointer<QObject> strong = weak;
-        QVERIFY(strong.isNull());
-    }
-
-    QCOMPARE(weak.data(), &obj);
-    // if something went wrong, we'll probably crash here
-}
-
-void tst_QSharedPointer::sharedPointerFromQObjectWithWeak()
-{
-    QObject *ptr = new QObject;
-    QWeakPointer<QObject> weak = ptr;
-    {
-        QSharedPointer<QObject> shared(ptr);
-        QVERIFY(!weak.isNull());
-        QCOMPARE(shared.data(), ptr);
-        QCOMPARE(weak.data(), ptr);
-    }
-    QVERIFY(weak.isNull());
-}
-
 void tst_QSharedPointer::weakQObjectFromSharedPointer()
 {
-    // this is the inverse of the above: you're allowed to create a QWeakPointer
-    // from a managed QObject
-    QSharedPointer<QObject> shared(new QObject);
-    QWeakPointer<QObject> weak = shared.data();
-    QVERIFY(!weak.isNull());
+    {
+        QSharedPointer<QObject> shared(new QObject);
+        QWeakPointer<QObject> weak = shared;
+        QVERIFY(!weak.isNull());
 
-    // delete:
-    shared.clear();
-    QVERIFY(weak.isNull());
+        // delete:
+        shared.clear();
+        QVERIFY(weak.isNull());
+    }
 }
 
 void tst_QSharedPointer::objectCast()
@@ -1122,6 +1061,60 @@ void tst_QSharedPointer::objectCast()
     safetyCheck();
 }
 
+
+void tst_QSharedPointer::objectCastStdSharedPtr()
+{
+    {
+        OtherObject *data = new OtherObject;
+        std::shared_ptr<QObject> baseptr = std::shared_ptr<QObject>(data);
+        QVERIFY(baseptr.get() == data);
+
+        // perform successful object cast
+        std::shared_ptr<OtherObject> ptr = qobject_pointer_cast<OtherObject>(baseptr);
+        QVERIFY(ptr.get());
+        QVERIFY(ptr.get() == data);
+
+        QVERIFY(baseptr.get() == data);
+    }
+
+    {
+        OtherObject *data = new OtherObject;
+        std::shared_ptr<QObject> baseptr = std::shared_ptr<QObject>(data);
+        QVERIFY(baseptr.get() == data);
+
+        // perform successful object cast
+        std::shared_ptr<OtherObject> ptr = qobject_pointer_cast<OtherObject>(std::move(baseptr));
+        QVERIFY(ptr.get());
+        QVERIFY(ptr.get() == data);
+
+        QVERIFY(!baseptr.get());
+    }
+
+    {
+        QObject *data = new QObject;
+        std::shared_ptr<QObject> baseptr = std::shared_ptr<QObject>(data);
+        QVERIFY(baseptr.get() == data);
+
+        // perform unsuccessful object cast
+        std::shared_ptr<OtherObject> ptr = qobject_pointer_cast<OtherObject>(baseptr);
+        QVERIFY(!ptr.get());
+
+        QVERIFY(baseptr.get() == data);
+    }
+
+    {
+        QObject *data = new QObject;
+        std::shared_ptr<QObject> baseptr = std::shared_ptr<QObject>(data);
+        QVERIFY(baseptr.get() == data);
+
+        // perform unsuccessful object cast
+        std::shared_ptr<OtherObject> ptr = qobject_pointer_cast<OtherObject>(std::move(baseptr));
+        QVERIFY(!ptr.get());
+
+        QVERIFY(baseptr.get() == data);
+    }
+}
+
 void tst_QSharedPointer::differentPointers()
 {
     {
@@ -1250,8 +1243,8 @@ void tst_QSharedPointer::dynamicCast()
         QCOMPARE(derivedptr.data(), aData);
         QCOMPARE(static_cast<Data *>(derivedptr.data()), baseptr.data());
     }
-    QCOMPARE(int(refCountData(baseptr)->weakref.load()), 1);
-    QCOMPARE(int(refCountData(baseptr)->strongref.load()), 1);
+    QCOMPARE(int(refCountData(baseptr)->weakref.loadRelaxed()), 1);
+    QCOMPARE(int(refCountData(baseptr)->strongref.loadRelaxed()), 1);
 
     {
         QWeakPointer<Data> weakptr = baseptr;
@@ -1260,8 +1253,8 @@ void tst_QSharedPointer::dynamicCast()
         QCOMPARE(derivedptr.data(), aData);
         QCOMPARE(static_cast<Data *>(derivedptr.data()), baseptr.data());
     }
-    QCOMPARE(int(refCountData(baseptr)->weakref.load()), 1);
-    QCOMPARE(int(refCountData(baseptr)->strongref.load()), 1);
+    QCOMPARE(int(refCountData(baseptr)->weakref.loadRelaxed()), 1);
+    QCOMPARE(int(refCountData(baseptr)->strongref.loadRelaxed()), 1);
 
     {
         QSharedPointer<DerivedData> derivedptr = baseptr.dynamicCast<DerivedData>();
@@ -1269,8 +1262,8 @@ void tst_QSharedPointer::dynamicCast()
         QCOMPARE(derivedptr.data(), aData);
         QCOMPARE(static_cast<Data *>(derivedptr.data()), baseptr.data());
     }
-    QCOMPARE(int(refCountData(baseptr)->weakref.load()), 1);
-    QCOMPARE(int(refCountData(baseptr)->strongref.load()), 1);
+    QCOMPARE(int(refCountData(baseptr)->weakref.loadRelaxed()), 1);
+    QCOMPARE(int(refCountData(baseptr)->strongref.loadRelaxed()), 1);
 }
 
 void tst_QSharedPointer::dynamicCastDifferentPointers()
@@ -1285,8 +1278,8 @@ void tst_QSharedPointer::dynamicCastDifferentPointers()
         QCOMPARE(derivedptr.data(), aData);
         QCOMPARE(static_cast<Data *>(derivedptr.data()), baseptr.data());
     }
-    QCOMPARE(int(refCountData(baseptr)->weakref.load()), 1);
-    QCOMPARE(int(refCountData(baseptr)->strongref.load()), 1);
+    QCOMPARE(int(refCountData(baseptr)->weakref.loadRelaxed()), 1);
+    QCOMPARE(int(refCountData(baseptr)->strongref.loadRelaxed()), 1);
 
     {
         QWeakPointer<Data> weakptr = baseptr;
@@ -1295,8 +1288,8 @@ void tst_QSharedPointer::dynamicCastDifferentPointers()
         QCOMPARE(derivedptr.data(), aData);
         QCOMPARE(static_cast<Data *>(derivedptr.data()), baseptr.data());
     }
-    QCOMPARE(int(refCountData(baseptr)->weakref.load()), 1);
-    QCOMPARE(int(refCountData(baseptr)->strongref.load()), 1);
+    QCOMPARE(int(refCountData(baseptr)->weakref.loadRelaxed()), 1);
+    QCOMPARE(int(refCountData(baseptr)->strongref.loadRelaxed()), 1);
 
     {
         QSharedPointer<DiffPtrDerivedData> derivedptr = baseptr.dynamicCast<DiffPtrDerivedData>();
@@ -1304,8 +1297,8 @@ void tst_QSharedPointer::dynamicCastDifferentPointers()
         QCOMPARE(derivedptr.data(), aData);
         QCOMPARE(static_cast<Data *>(derivedptr.data()), baseptr.data());
     }
-    QCOMPARE(int(refCountData(baseptr)->weakref.load()), 1);
-    QCOMPARE(int(refCountData(baseptr)->strongref.load()), 1);
+    QCOMPARE(int(refCountData(baseptr)->weakref.loadRelaxed()), 1);
+    QCOMPARE(int(refCountData(baseptr)->strongref.loadRelaxed()), 1);
 
     {
         Stuffing *nakedptr = dynamic_cast<Stuffing *>(baseptr.data());
@@ -1330,8 +1323,8 @@ void tst_QSharedPointer::dynamicCastVirtualBase()
         QCOMPARE(derivedptr.data(), aData);
         QCOMPARE(static_cast<Data *>(derivedptr.data()), baseptr.data());
     }
-    QCOMPARE(int(refCountData(baseptr)->weakref.load()), 1);
-    QCOMPARE(int(refCountData(baseptr)->strongref.load()), 1);
+    QCOMPARE(int(refCountData(baseptr)->weakref.loadRelaxed()), 1);
+    QCOMPARE(int(refCountData(baseptr)->strongref.loadRelaxed()), 1);
 
     {
         QWeakPointer<Data> weakptr = baseptr;
@@ -1340,8 +1333,8 @@ void tst_QSharedPointer::dynamicCastVirtualBase()
         QCOMPARE(derivedptr.data(), aData);
         QCOMPARE(static_cast<Data *>(derivedptr.data()), baseptr.data());
     }
-    QCOMPARE(int(refCountData(baseptr)->weakref.load()), 1);
-    QCOMPARE(int(refCountData(baseptr)->strongref.load()), 1);
+    QCOMPARE(int(refCountData(baseptr)->weakref.loadRelaxed()), 1);
+    QCOMPARE(int(refCountData(baseptr)->strongref.loadRelaxed()), 1);
 
     {
         QSharedPointer<VirtualDerived> derivedptr = baseptr.dynamicCast<VirtualDerived>();
@@ -1349,8 +1342,8 @@ void tst_QSharedPointer::dynamicCastVirtualBase()
         QCOMPARE(derivedptr.data(), aData);
         QCOMPARE(static_cast<Data *>(derivedptr.data()), baseptr.data());
     }
-    QCOMPARE(int(refCountData(baseptr)->weakref.load()), 1);
-    QCOMPARE(int(refCountData(baseptr)->strongref.load()), 1);
+    QCOMPARE(int(refCountData(baseptr)->weakref.loadRelaxed()), 1);
+    QCOMPARE(int(refCountData(baseptr)->strongref.loadRelaxed()), 1);
 }
 
 void tst_QSharedPointer::dynamicCastFailure()
@@ -1362,15 +1355,15 @@ void tst_QSharedPointer::dynamicCastFailure()
         QSharedPointer<DerivedData> derivedptr = qSharedPointerDynamicCast<DerivedData>(baseptr);
         QVERIFY(derivedptr.isNull());
     }
-    QCOMPARE(int(refCountData(baseptr)->weakref.load()), 1);
-    QCOMPARE(int(refCountData(baseptr)->strongref.load()), 1);
+    QCOMPARE(int(refCountData(baseptr)->weakref.loadRelaxed()), 1);
+    QCOMPARE(int(refCountData(baseptr)->strongref.loadRelaxed()), 1);
 
     {
         QSharedPointer<DerivedData> derivedptr = baseptr.dynamicCast<DerivedData>();
         QVERIFY(derivedptr.isNull());
     }
-    QCOMPARE(int(refCountData(baseptr)->weakref.load()), 1);
-    QCOMPARE(int(refCountData(baseptr)->strongref.load()), 1);
+    QCOMPARE(int(refCountData(baseptr)->weakref.loadRelaxed()), 1);
+    QCOMPARE(int(refCountData(baseptr)->strongref.loadRelaxed()), 1);
 }
 
 void tst_QSharedPointer::dynamicCastFailureNoLeak()
@@ -1677,7 +1670,6 @@ void tst_QSharedPointer::customDeleter()
     safetyCheck();
 }
 
-#ifdef Q_COMPILER_LAMBDA
 // The compiler needs to be in C++11 mode and to support lambdas
 void tst_QSharedPointer::lambdaCustomDeleter()
 {
@@ -1705,7 +1697,33 @@ void tst_QSharedPointer::lambdaCustomDeleter()
     }
     safetyCheck();
 }
-#endif
+
+void tst_QSharedPointer::customDeleterOnNullptr()
+{
+    Data *null = nullptr;
+    int callCount = 0;
+    auto deleter = [&callCount](Data *) { ++callCount; };
+    {
+        QSharedPointer<Data> ptr(null, deleter);
+    }
+    safetyCheck();
+    QCOMPARE(callCount, 1);
+
+    callCount = 0;
+    {
+        QSharedPointer<Data> ptr(nullptr, deleter);
+    }
+    safetyCheck();
+    QCOMPARE(callCount, 1);
+
+    callCount = 0;
+    {
+        QSharedPointer<Data> ptr1(null, deleter);
+        QSharedPointer<Data> ptr2(nullptr, deleter);
+    }
+    safetyCheck();
+    QCOMPARE(callCount, 2);
+}
 
 void customQObjectDeleterFn(QObject *obj)
 {
@@ -1741,8 +1759,8 @@ void tst_QSharedPointer::creating()
         QCOMPARE(Data::destructorCounter, 1);
 
         // valgrind will complain here if something happened to the pointer
-        QVERIFY(d->weakref.load() == 1);
-        QVERIFY(d->strongref.load() == 0);
+        QVERIFY(d->weakref.loadRelaxed() == 1);
+        QVERIFY(d->strongref.loadRelaxed() == 0);
     }
     safetyCheck();
 
@@ -1975,9 +1993,9 @@ void tst_QSharedPointer::threadStressTest()
     memset(guard2, 0, sizeof guard2);
 
     for (int r = 0; r < 5; ++r) {
-        QVector<QThread*> allThreads(6 * qMax(strongThreadCount, weakThreadCount) + 3, 0);
+        QList<QThread*> allThreads(6 * qMax(strongThreadCount, weakThreadCount) + 3, 0);
         QSharedPointer<ThreadData> base = QSharedPointer<ThreadData>(new ThreadData(&counter));
-        counter.store(0);
+        counter.storeRelaxed(0);
 
         // set the pointers
         for (int i = 0; i < strongThreadCount; ++i) {
@@ -2011,12 +2029,12 @@ void tst_QSharedPointer::threadStressTest()
         // verify that the count is the right range
         int minValue = strongThreadCount;
         int maxValue = strongThreadCount + weakThreadCount;
-        QVERIFY(counter.load() >= minValue);
-        QVERIFY(counter.load() <= maxValue);
+        QVERIFY(counter.loadRelaxed() >= minValue);
+        QVERIFY(counter.loadRelaxed() <= maxValue);
     }
 }
 
-template<typename Container, bool Ordered>
+template<typename Container, bool Ordered, bool Multi>
 void hashAndMapTest()
 {
     typedef typename Container::key_type Key;
@@ -2061,26 +2079,30 @@ void hashAndMapTest()
         QVERIFY(it == c.end());
     }
 
-    c.insertMulti(k1, Value(47));
-    it = c.find(k1);
-    QVERIFY(it != c.end());
-    QCOMPARE(it.key(), k1);
-    ++it;
-    QVERIFY(it != c.end());
-    QCOMPARE(it.key(), k1);
-    ++it;
-    if (Ordered)
-        QVERIFY(it == c.end());
+    if (Multi) {
+        c.insert(k1, Value(47));
+        it = c.find(k1);
+        QVERIFY(it != c.end());
+        QCOMPARE(it.key(), k1);
+        ++it;
+        QVERIFY(it != c.end());
+        QCOMPARE(it.key(), k1);
+        ++it;
+        if (Ordered)
+            QVERIFY(it == c.end());
+    }
 }
 
 void tst_QSharedPointer::map()
 {
-    hashAndMapTest<QMap<QSharedPointer<int>, int>, true>();
+    hashAndMapTest<QMap<QSharedPointer<int>, int>, true, false>();
+    hashAndMapTest<QMultiMap<QSharedPointer<int>, int>, true, true>();
 }
 
 void tst_QSharedPointer::hash()
 {
-    hashAndMapTest<QHash<QSharedPointer<int>, int>, false>();
+    hashAndMapTest<QHash<QSharedPointer<int>, int>, false, false>();
+    hashAndMapTest<QMultiHash<QSharedPointer<int>, int>, false, true>();
 }
 
 void tst_QSharedPointer::validConstructs()
@@ -2229,12 +2251,12 @@ void tst_QSharedPointer::invalidConstructs_data()
 
     QTest::newRow("weak-pointer-from-regular-pointer")
         << &QTest::QExternalTest::tryCompileFail
-        << "Data *ptr = 0;\n"
+        << "Data *ptr = nullptr;\n"
            "QWeakPointer<Data> weakptr(ptr);\n";
 
     QTest::newRow("shared-pointer-implicit-from-uninitialized")
         << &QTest::QExternalTest::tryCompileFail
-        << "Data *ptr = 0;\n"
+        << "Data *ptr = nullptr;\n"
            "QSharedPointer<Data> weakptr = Qt::Uninitialized;\n";
 
     QTest::newRow("incompatible-custom-deleter1")
@@ -2245,11 +2267,14 @@ void tst_QSharedPointer::invalidConstructs_data()
         << &QTest::QExternalTest::tryCompileFail
         << "struct IncompatibleCustomDeleter { void operator()(int *); };\n"
            "QSharedPointer<Data> ptr(new Data, IncompatibleCustomDeleter());\n";
-#ifdef Q_COMPILER_LAMBDA
     QTest::newRow("incompatible-custom-lambda-deleter")
         << &QTest::QExternalTest::tryCompileFail
         << "QSharedPointer<Data> ptr(new Data, [](int *) {});\n";
-#endif
+
+    QTest::newRow("incompatible-overload")
+        << &QTest::QExternalTest::tryCompileFail
+        << "void foo(QSharedPointer<DerivedData>) {}\n"
+           "void bar() { foo(QSharedPointer<Data>()); }\n";
 }
 
 void tst_QSharedPointer::invalidConstructs()
@@ -2332,50 +2357,6 @@ void tst_QSharedPointer::qvariantCast()
     }
     // Intentionally does not compile.
 //     QSharedPointer<int> sop = qSharedPointerFromVariant<int>(v);
-
-    v = QVariant::fromValue(sp.toWeakRef());
-
-    {
-        QWeakPointer<QObject> other = qWeakPointerFromVariant<QObject>(v);
-        QCOMPARE(other.data()->objectName(), QString::fromLatin1("A test name"));
-    }
-    {
-        QWeakPointer<QIODevice> other = qWeakPointerFromVariant<QIODevice>(v);
-        QCOMPARE(other.data()->objectName(), QString::fromLatin1("A test name"));
-    }
-    {
-        QWeakPointer<QFile> other = qWeakPointerFromVariant<QFile>(v);
-        QCOMPARE(other.data()->objectName(), QString::fromLatin1("A test name"));
-    }
-    {
-        QWeakPointer<QThread> other = qWeakPointerFromVariant<QThread>(v);
-        QVERIFY(!other);
-    }
-
-    // Intentionally does not compile.
-//     QWeakPointer<int> sop = qWeakPointerFromVariant<int>(v);
-
-    QFile file;
-    QWeakPointer<QFile> tracking = &file;
-    tracking.data()->setObjectName("A test name");
-    v = QVariant::fromValue(tracking);
-
-    {
-        QWeakPointer<QObject> other = qWeakPointerFromVariant<QObject>(v);
-        QCOMPARE(other.data()->objectName(), QString::fromLatin1("A test name"));
-    }
-    {
-        QWeakPointer<QIODevice> other = qWeakPointerFromVariant<QIODevice>(v);
-        QCOMPARE(other.data()->objectName(), QString::fromLatin1("A test name"));
-    }
-    {
-        QWeakPointer<QFile> other = qWeakPointerFromVariant<QFile>(v);
-        QCOMPARE(other.data()->objectName(), QString::fromLatin1("A test name"));
-    }
-    {
-        QWeakPointer<QThread> other = qWeakPointerFromVariant<QThread>(v);
-        QVERIFY(!other);
-    }
 }
 
 class SomeClass : public QEnableSharedFromThis<SomeClass>
@@ -2748,5 +2729,50 @@ void tst_QSharedPointer::reentrancyWhileDestructing()
     ReentrancyWhileDestructing::A obj;
 }
 
+namespace {
+struct Base1 {};
+struct Base2 {};
+
+struct Child1 : Base1 {};
+struct Child2 : Base2 {};
+
+template<template<typename> class SmartPtr>
+struct Overloaded
+{
+    std::array<int, 1> call(const SmartPtr<const Base1> &)
+    {
+        return {};
+    }
+    std::array<int, 2> call(const SmartPtr<const Base2> &)
+    {
+        return {};
+    }
+    static const constexpr uint base1Called = sizeof(std::array<int, 1>);
+    static const constexpr uint base2Called = sizeof(std::array<int, 2>);
+
+    void test()
+    {
+#define QVERIFY_CALLS(expr, base) static_assert(sizeof(call(expr)) == base##Called)
+        QVERIFY_CALLS(SmartPtr<Base1>{}, base1);
+        QVERIFY_CALLS(SmartPtr<Base2>{}, base2);
+        QVERIFY_CALLS(SmartPtr<const Base1>{}, base1);
+        QVERIFY_CALLS(SmartPtr<const Base2>{}, base2);
+        QVERIFY_CALLS(SmartPtr<Child1>{}, base1);
+        QVERIFY_CALLS(SmartPtr<Child2>{}, base2);
+        QVERIFY_CALLS(SmartPtr<const Child1>{}, base1);
+        QVERIFY_CALLS(SmartPtr<const Child2>{}, base2);
+#undef QVERIFY_CALLS
+    }
+};
+}
+
+void tst_QSharedPointer::overloads()
+{
+    Overloaded<QSharedPointer> sharedOverloaded;
+    sharedOverloaded.test();
+    Overloaded<QWeakPointer> weakOverloaded;
+    weakOverloaded.test();
+}
+
 QTEST_MAIN(tst_QSharedPointer)
 #include "tst_qsharedpointer.moc"
